@@ -1,16 +1,29 @@
-# Copyright (C) 2017 MongoDB Inc.
+# Copyright (C) 2018-present MongoDB, Inc.
 #
-# This program is free software: you can redistribute it and/or  modify
-# it under the terms of the GNU Affero General Public License, version 3,
-# as published by the Free Software Foundation.
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the Server Side Public License, version 1,
+# as published by MongoDB, Inc.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+# Server Side Public License for more details.
 #
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the Server Side Public License
+# along with this program. If not, see
+# <http://www.mongodb.com/licensing/server-side-public-license>.
+#
+# As a special exception, the copyright holders give permission to link the
+# code of portions of this program with the OpenSSL library under certain
+# conditions as described in each individual source file and distribute
+# linked combinations including the program with the OpenSSL library. You
+# must comply with the Server Side Public License in all respects for
+# all of the code used other than as permitted herein. If you modify file(s)
+# with this exception, you may extend this exception to your version of the
+# file(s), but you are not obligated to do so. If you do not wish to do so,
+# delete this exception statement from your version. If you delete this
+# exception statement from all source files in the program, then also delete
+# it in the license file.
 #
 """
 Common error handling code for IDL compiler.
@@ -85,6 +98,7 @@ ERROR_ID_IS_NODE_VALID_INT = "ID0049"
 ERROR_ID_IS_NODE_VALID_NON_NEGATIVE_INT = "ID0050"
 ERROR_ID_IS_DUPLICATE_COMPARISON_ORDER = "ID0051"
 ERROR_ID_IS_COMMAND_TYPE_EXTRANEOUS = "ID0052"
+ERROR_ID_NON_CONST_GETTER_IN_IMMUTABLE_STRUCT = "ID0069"
 
 
 class IDLError(Exception):
@@ -617,6 +631,16 @@ class ParserContext(object):
         self._add_error(location, ERROR_ID_COMMAND_DUPLICATES_FIELD,
                         ("Command '%s' cannot have the same name as a field.") % (command_name))
 
+    def add_bad_field_non_const_getter_in_immutable_struct_error(self, location, struct_name,
+                                                                 field_name):
+        # type: (common.SourceLocation, unicode, unicode) -> None
+        """Add an error about marking a field with non_const_getter in an immutable struct."""
+        # pylint: disable=invalid-name
+        self._add_error(location, ERROR_ID_NON_CONST_GETTER_IN_IMMUTABLE_STRUCT,
+                        ("Cannot generate a non-const getter for field '%s' in struct '%s' since"
+                         " struct '%s' is marked as immutable.") % (field_name, struct_name,
+                                                                    struct_name))
+
     def is_scalar_non_negative_int_node(self, node, node_name):
         # type: (Union[yaml.nodes.MappingNode, yaml.nodes.ScalarNode, yaml.nodes.SequenceNode], unicode) -> bool
         """Return True if this YAML node is a Scalar and a valid non-negative int."""
@@ -676,7 +700,7 @@ def _assert_unique_error_messages():
 
     error_ids_set = set(error_ids)
     if len(error_ids) != len(error_ids_set):
-        raise IDLError("IDL error codes prefixed with ERRROR_ID are not unique.")
+        raise IDLError("IDL error codes prefixed with ERROR_ID are not unique.")
 
 
 # On file import, check the error messages are unique
