@@ -24,34 +24,41 @@ var checkLog;
             return assert.commandWorked(cmdRes).log;
         };
 
+        /*
+         * Calls the 'getLog' function on the provided connection 'conn' to see if the provided msg
+         * is found in the logs. Note: this function does not throw an exception, so the return
+         * value should not be ignored.
+         */
+        const checkContainsOnce = function(conn, msg) {
+            const logMessages = getGlobalLog(conn);
+            if (logMessages === null) {
+                return false;
+            }
+            for (let logMsg of logMessages) {
+                if (logMsg.includes(msg)) {
+                    return true;
+                }
+            }
+            return false;
+        };
+
         /*
          * Calls the 'getLog' function at regular intervals on the provided connection 'conn' until
-         * the provided 'msg' is found in the logs, or 5 minutes have elapsed. Throws an exception
-         * on timeout.
+         * the provided 'msg' is found in the logs, or it times out. Throws an exception on timeout.
          */
         var contains = function(conn, msg, timeout = 5 * 60 * 1000) {
             assert.soon(function() {
-                var logMessages = getGlobalLog(conn);
-                if (logMessages === null) {
-                    return false;
-                }
-                for (var i = 0; i < logMessages.length; i++) {
-                    if (logMessages[i].indexOf(msg) != -1) {
-                        return true;
-                    }
-                }
-                return false;
+                return checkContainsOnce(conn, msg);
             }, 'Could not find log entries containing the following message: ' + msg, timeout, 300);
         };
 
         /*
          * Calls the 'getLog' function at regular intervals on the provided connection 'conn' until
-         * the provided 'msg' is found in the logs exactly 'expectedCount' times, or 5 minutes have
-         * elapsed.
+         * the provided 'msg' is found in the logs exactly 'expectedCount' times, or it times out.
          * Throws an exception on timeout.
          */
-        var containsWithCount = function(conn, msg, expectedCount) {
-            let count;
+        var containsWithCount = function(conn, msg, expectedCount, timeout = 5 * 60 * 1000) {
+            var count = 0;
             assert.soon(
                 function() {
                     count = 0;
@@ -67,9 +74,9 @@ var checkLog;
 
                     return expectedCount === count;
                 },
-                'Expected ' + expectedCount + ' log entries containing the following message: ' +
-                    msg + ' on node ' + conn.name,
-                5 * 60 * 1000,
+                'Expected ' + expectedCount + ', but instead saw ' + count +
+                    ' log entries containing the following message: ' + msg,
+                timeout,
                 300);
         };
 
@@ -104,6 +111,7 @@ var checkLog;
 
         return {
             getGlobalLog: getGlobalLog,
+            checkContainsOnce: checkContainsOnce,
             contains: contains,
             containsWithCount: containsWithCount,
             formatAsLogLine: formatAsLogLine
