@@ -41,6 +41,13 @@
                 version is ${glibc_version}, skipping this test.`);
             return;
         } else {
+            var verCheck =
+                runProgram("grep", "/etc/os-release", "\"SUSE Linux Enterprise Server\"");
+            if (verCheck == 0) {
+                jsTestLog(`HOSTALIASES does not seem to work as expected but we detected SLES. GLIBC
+                    version is ${glibc_version}, skipping this test.`);
+                return;
+            }
             assert(false, `HOSTALIASES does not seem to work as expected on this system. GLIBC
                 version is ${glibc_version}`);
         }
