@@ -40,6 +40,7 @@
 #include <boost/algorithm/string/classification.hpp>
 #include <boost/algorithm/string/find_iterator.hpp>
 #include <boost/algorithm/string/predicate.hpp>
+#include <boost/range/algorithm/count.hpp>
 
 #include "mongo/base/status_with.h"
 #include "mongo/bson/bsonobjbuilder.h"
@@ -57,8 +58,6 @@ using namespace std::literals::string_literals;
 namespace {
 constexpr std::array<char, 16> hexits{
     '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
-const mongo::StringData kURIPrefix{"mongodb://"};
-const mongo::StringData kURISRVPrefix{"mongodb+srv://"};
 
 // This vector must remain sorted.  It is over pairs to facilitate a call to `std::includes` using
 // a `std::map<std::string, std::string>` as the other parameter.
@@ -87,13 +86,20 @@ void mongo::uriEncode(std::ostream& ss, StringData toEncode, StringData passthro
 mongo::StatusWith<std::string> mongo::uriDecode(StringData toDecode) {
     StringBuilder out;
     for (size_t i = 0; i < toDecode.size(); ++i) {
-        const auto c = toDecode[i];
+        const char c = toDecode[i];
         if (c == '%') {
             if (i + 2 > toDecode.size()) {
                 return Status(ErrorCodes::FailedToParse,
                               "Encountered partial escape sequence at end of string");
             }
-            out << fromHex(toDecode.substr(i + 1, 2));
+            auto swHex = fromHex(toDecode.substr(i + 1, 2));
+            if (swHex.isOK()) {
+                out << swHex.getValue();
+            } else {
+                return Status(ErrorCodes::Error(51040),
+                              "The characters after the % do not form a hex value. Please escape "
+                              "the % or pass a valid hex value. ");
+            }
             i += 2;
         } else {
             out << c;
@@ -106,6 +112,10 @@ namespace mongo {
 
 namespace {
 
+constexpr StringData kURIPrefix = "mongodb://"_sd;
+constexpr StringData kURISRVPrefix = "mongodb+srv://"_sd;
+constexpr StringData kDefaultMongoHost = "127.0.0.1:27017"_sd;
+
 /**
  * Helper Method for MongoURI::parse() to split a string into exactly 2 pieces by a char
  * delimeter.
@@ -173,14 +183,11 @@ std::map<std::string, std::string> parseOptions(StringData options, StringData u
                           << "Missing a key for key/value pair in the options for mongodb:// URL: "
                           << url);
         }
-        const auto key = uriDecode(keyRaw);
-        if (!key.isOK()) {
-            uasserted(
-                ErrorCodes::FailedToParse,
-                str::stream() << "Key '" << keyRaw
-                              << "' in options cannot properly be URL decoded for mongodb:// URL: "
-                              << url);
-        }
+        const auto key = uassertStatusOKWithContext(
+            uriDecode(keyRaw),
+            str::stream() << "Key '" << keyRaw
+                          << "' in options cannot properly be URL decoded for mongodb:// URL: "
+                          << url);
         const auto valRaw = kvPair.second;
         if (valRaw.empty()) {
             uasserted(ErrorCodes::FailedToParse,
@@ -188,16 +195,13 @@ std::map<std::string, std::string> parseOptions(StringData options, StringData u
                                     << "' in the options for mongodb:// URL: "
                                     << url);
         }
-        const auto val = uriDecode(valRaw);
-        if (!val.isOK()) {
-            uasserted(
-                ErrorCodes::FailedToParse,
-                str::stream() << "Value '" << valRaw << "' for key '" << keyRaw
-                              << "' in options cannot properly be URL decoded for mongodb:// URL: "
-                              << url);
-        }
+        const auto val = uassertStatusOKWithContext(
+            uriDecode(valRaw),
+            str::stream() << "Value '" << valRaw << "' for key '" << keyRaw
+                          << "' in options cannot properly be URL decoded for mongodb:// URL: "
+                          << url);
 
-        ret[key.getValue()] = val.getValue();
+        ret[key] = val;
     }
 
     return ret;
@@ -350,21 +354,14 @@ MongoURI MongoURI::parseImpl(const std::string& url) {
     }
 
     // Get the username and make sure it did not fail to decode
-    const auto usernameWithStatus = uriDecode(usernameSD);
-    if (!usernameWithStatus.isOK()) {
-        uasserted(ErrorCodes::FailedToParse,
-                  str::stream() << "Username cannot properly be URL decoded for mongodb:// URL: "
-                                << url);
-    }
-    const auto username = usernameWithStatus.getValue();
+    const auto username = uassertStatusOKWithContext(
+        uriDecode(usernameSD),
+        str::stream() << "Username cannot properly be URL decoded for mongodb:// URL: " << url);
 
     // Get the password and make sure it did not fail to decode
-    const auto passwordWithStatus = uriDecode(passwordSD);
-    if (!passwordWithStatus.isOK())
-        uasserted(ErrorCodes::FailedToParse,
-                  str::stream() << "Password cannot properly be URL decoded for mongodb:// URL: "
-                                << url);
-    const auto password = passwordWithStatus.getValue();
+    const auto password = uassertStatusOKWithContext(
+        uriDecode(passwordSD),
+        str::stream() << "Password cannot properly be URL decoded for mongodb:// URL: " << url);
 
     // 4. Validate, split, and URL decode the host identifiers.
     const auto hostIdentifiersStr = hostIdentifiers.toString();
@@ -373,14 +370,10 @@ MongoURI MongoURI::parseImpl(const std::string& url) {
                                              boost::first_finder(",", boost::is_iequal()));
          i != std::remove_reference<decltype((i))>::type{};
          ++i) {
-        const auto hostWithStatus = uriDecode(boost::copy_range<std::string>(*i));
-        if (!hostWithStatus.isOK()) {
-            uasserted(ErrorCodes::FailedToParse,
-                      str::stream() << "Host cannot properly be URL decoded for mongodb:// URL: "
-                                    << url);
-        }
+        const auto host = uassertStatusOKWithContext(
+            uriDecode(boost::copy_range<std::string>(*i)),
+            str::stream() << "Host cannot properly be URL decoded for mongodb:// URL: " << url);
 
-        const auto host = hostWithStatus.getValue();
         if (host.empty()) {
             continue;
         }
@@ -443,14 +436,11 @@ MongoURI MongoURI::parseImpl(const std::string& url) {
     }
 
     // 5. Decode the database name
-    const auto databaseWithStatus = uriDecode(databaseSD);
-    if (!databaseWithStatus.isOK()) {
-        uasserted(ErrorCodes::FailedToParse,
-                  str::stream() << "Database name cannot properly be URL "
-                                   "decoded for mongodb:// URL: "
-                                << url);
-    }
-    const auto database = databaseWithStatus.getValue();
+    const auto database =
+        uassertStatusOKWithContext(uriDecode(databaseSD),
+                                   str::stream() << "Database name cannot properly be URL "
+                                                    "decoded for mongodb:// URL: "
+                                                 << url);
 
     // 6. Validate the database contains no prohibited characters
     // Prohibited characters:
@@ -497,10 +487,28 @@ MongoURI MongoURI::parseImpl(const std::string& url) {
         }
     }
 
+    transport::ConnectSSLMode sslMode = transport::kGlobalSSLMode;
+    auto sslModeIter = options.find("ssl");
+    if (sslModeIter != options.end()) {
+        const auto& val = sslModeIter->second;
+        if (val == "true") {
+            sslMode = transport::kEnableSSL;
+        } else if (val == "false") {
+            sslMode = transport::kDisableSSL;
+        } else {
+            uasserted(51041, str::stream() << "ssl must be either 'true' or 'false', not" << val);
+        }
+    }
+
     ConnectionString cs(
         setName.empty() ? ConnectionString::MASTER : ConnectionString::SET, servers, setName);
-    return MongoURI(
-        std::move(cs), username, password, database, std::move(retryWrites), std::move(options));
+    return MongoURI(std::move(cs),
+                    username,
+                    password,
+                    database,
+                    std::move(retryWrites),
+                    sslMode,
+                    std::move(options));
 }
 
 StatusWith<MongoURI> MongoURI::parse(const std::string& url) try {
@@ -513,8 +521,53 @@ const boost::optional<std::string> MongoURI::getAppName() const {
     const auto optIter = _options.find("appName");
     if (optIter != end(_options)) {
         return optIter->second;
-    } else {
-        return boost::none;
     }
+    return boost::none;
+}
+
+std::string MongoURI::canonicalizeURIAsString() const {
+    StringBuilder uri;
+    uri << kURIPrefix;
+    if (!_user.empty()) {
+        uri << uriEncode(_user);
+        if (!_password.empty()) {
+            uri << ":" << uriEncode(_password);
+        }
+        uri << "@";
+    }
+
+    const auto& servers = _connectString.getServers();
+    if (!servers.empty()) {
+        auto delimeter = "";
+        for (auto& hostAndPort : servers) {
+            if (boost::count(hostAndPort.host(), ':') > 1) {
+                uri << delimeter << "[" << uriEncode(hostAndPort.host()) << "]"
+                    << ":" << uriEncode(std::to_string(hostAndPort.port()));
+            } else if (StringData(hostAndPort.host()).endsWith(".sock")) {
+                uri << delimeter << uriEncode(hostAndPort.host());
+            } else {
+                uri << delimeter << uriEncode(hostAndPort.host()) << ":"
+                    << uriEncode(std::to_string(hostAndPort.port()));
+            }
+            delimeter = ",";
+        }
+    } else {
+        uri << kDefaultMongoHost;
+    }
+
+    uri << "/";
+    if (!_database.empty()) {
+        uri << uriEncode(_database);
+    }
+
+    if (!_options.empty()) {
+        auto delimeter = "";
+        uri << "?";
+        for (const auto& pair : _options) {
+            uri << delimeter << uriEncode(pair.first) << "=" << uriEncode(pair.second);
+            delimeter = "&";
+        }
+    }
+    return uri.str();
 }
 }  // namespace mongo
