@@ -117,7 +117,8 @@ Status renameCollectionCommon(OperationContext* opCtx,
                               const NamespaceString& target,
                               OptionalCollectionUUID targetUUID,
                               repl::OpTime renameOpTimeFromApplyOps,
-                              const RenameCollectionOptions& options) {
+                              const RenameCollectionOptions& options,
+                              bool forApplyOps) {
     // A valid 'renameOpTimeFromApplyOps' is not allowed when writes are replicated.
     if (!renameOpTimeFromApplyOps.isNull() && opCtx->writesAreReplicated()) {
         return Status(
@@ -216,14 +217,15 @@ Status renameCollectionCommon(OperationContext* opCtx,
             }
         }
 
-        if (!options.dropTarget) {
+        // RenameCollectionForCommand cannot drop target by renaming.
+        if (!forApplyOps && !options.dropTarget) {
             return Status(ErrorCodes::NamespaceExists, "target namespace exists");
         }
 
         // If UUID doesn't point to the existing target, we should rename the target rather than
         // drop it.
-        if (options.dropTargetUUID && options.dropTargetUUID != targetColl->uuid()) {
-            auto dropTargetNssFromUUID = getNamespaceFromUUID(opCtx, options.dropTargetUUID.get());
+        if (!options.dropTarget ||
+            (options.dropTargetUUID && options.dropTargetUUID != targetColl->uuid())) {
             // We need to rename the targetColl to a temporary name.
             auto status = renameTargetCollectionToTmp(
                 opCtx, source, targetUUID.get(), targetDB, target, targetColl->uuid().get());
@@ -520,7 +522,7 @@ Status renameCollectionCommon(OperationContext* opCtx,
     // in-place rename and remove the source collection.
     invariant(tmpName.db() == target.db());
     status = renameCollectionCommon(
-        opCtx, tmpName, target, targetUUID, renameOpTimeFromApplyOps, options);
+        opCtx, tmpName, target, targetUUID, renameOpTimeFromApplyOps, options, false);
     if (!status.isOK()) {
         return status;
     }
@@ -552,7 +554,7 @@ Status renameCollection(OperationContext* opCtx,
     log() << "renameCollectionForCommand: rename " << source << " to " << target << dropTargetMsg;
 
     OptionalCollectionUUID noTargetUUID;
-    return renameCollectionCommon(opCtx, source, target, noTargetUUID, {}, options);
+    return renameCollectionCommon(opCtx, source, target, noTargetUUID, {}, options, false);
 }
 
 
@@ -636,7 +638,8 @@ Status renameCollectionForApplyOps(OperationContext* opCtx,
           << targetNss << dropTargetMsg;
 
     options.stayTemp = cmd["stayTemp"].trueValue();
-    return renameCollectionCommon(opCtx, sourceNss, targetNss, targetUUID, renameOpTime, options);
+    return renameCollectionCommon(
+        opCtx, sourceNss, targetNss, targetUUID, renameOpTime, options, true);
 }
 
 Status renameCollectionForRollback(OperationContext* opCtx,
@@ -657,7 +660,7 @@ Status renameCollectionForRollback(OperationContext* opCtx,
     log() << "renameCollectionForRollback: rename " << source << " (" << uuid << ") to " << target
           << ".";
 
-    return renameCollectionCommon(opCtx, source, target, uuid, {}, options);
+    return renameCollectionCommon(opCtx, source, target, uuid, {}, options, false);
 }
 
 }  // namespace mongo
