@@ -49,7 +49,7 @@ public:
 
     void debugString(StringBuilder& debug, int level) const final;
 
-    void serialize(BSONObjBuilder* out) const final;
+    BSONObj getSerializedRightHandSide() const final;
 
     bool equivalent(const MatchExpression* other) const final;
 
