@@ -1,29 +1,31 @@
+
 /**
- * Copyright (C) 2017 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- * This program is free software: you can redistribute it and/or  modify
- * it under the terms of the GNU Affero General Public License, version 3,
- * as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
+ *    This program is distributed in the hope that it will be useful,
+ *    but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *    Server Side Public License for more details.
  *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
- * As a special exception, the copyright holders give permission to link the
- * code of portions of this program with the OpenSSL library under certain
- * conditions as described in each individual source file and distribute
- * linked combinations including the program with the OpenSSL library. You
- * must comply with the GNU Affero General Public License in all respects
- * for all of the code used other than as permitted herein. If you modify
- * file(s) with this exception, you may extend this exception to your
- * version of the file(s), but you are not obligated to do so. If you do not
- * wish to do so, delete this exception statement from your version. If you
- * delete this exception statement from all source files in the program,
- * then also delete it in the license file.
+ *    As a special exception, the copyright holders give permission to link the
+ *    code of portions of this program with the OpenSSL library under certain
+ *    conditions as described in each individual source file and distribute
+ *    linked combinations including the program with the OpenSSL library. You
+ *    must comply with the Server Side Public License in all respects for
+ *    all of the code used other than as permitted herein. If you modify file(s)
+ *    with this exception, you may extend this exception to your version of the
+ *    file(s), but you are not obligated to do so. If you do not wish to do so,
+ *    delete this exception statement from your version. If you delete this
+ *    exception statement from all source files in the program, then also delete
+ *    it in the license file.
  */
 
 #include "mongo/platform/basic.h"
@@ -68,7 +70,7 @@ protected:
         Timestamp ts, int version, std::size_t applyOpsIndex, Document docKey, UUID uuid) {
         _mock->queue.push_back(
             Document{{"_id",
-                      ResumeToken(ResumeTokenData(ts, version, applyOpsIndex, Value(docKey), uuid))
+                      ResumeToken(ResumeTokenData(ts, version, applyOpsIndex, uuid, Value(docKey)))
                           .toDocument(ResumeToken::SerializationFormat::kHexString)}});
     }
 
@@ -101,9 +103,8 @@ protected:
         std::size_t applyOpsIndex,
         boost::optional<Document> docKey,
         UUID uuid) {
-        ResumeToken token(
-            ResumeTokenData(ts, version, applyOpsIndex, docKey ? Value(*docKey) : Value(), uuid));
-        auto checkResumeToken = DocumentSourceEnsureResumeTokenPresent::create(getExpCtx(), token);
+        auto checkResumeToken = DocumentSourceEnsureResumeTokenPresent::create(
+            getExpCtx(), {ts, version, applyOpsIndex, uuid, docKey ? Value(*docKey) : Value()});
         checkResumeToken->setSource(_mock.get());
         return checkResumeToken;
     }
@@ -140,11 +141,17 @@ protected:
 
 class ShardCheckResumabilityTest : public CheckResumeTokenTest {
 protected:
-    intrusive_ptr<DocumentSourceShardCheckResumability> createShardCheckResumability(Timestamp ts) {
-        auto shardCheckResumability = DocumentSourceShardCheckResumability::create(getExpCtx(), ts);
+    intrusive_ptr<DocumentSourceShardCheckResumability> createShardCheckResumability(
+        ResumeTokenData tokenData) {
+        auto shardCheckResumability =
+            DocumentSourceShardCheckResumability::create(getExpCtx(), tokenData);
         shardCheckResumability->setSource(_mock.get());
         return shardCheckResumability;
     }
+    intrusive_ptr<DocumentSourceShardCheckResumability> createShardCheckResumability(Timestamp ts) {
+        return createShardCheckResumability(
+            ResumeToken::makeHighWaterMarkToken(ts, testUuid()).getData());
+    }
 };
 
 TEST_F(CheckResumeTokenTest, ShouldSucceedWithOnlyResumeToken) {
@@ -511,7 +518,9 @@ TEST_F(ShardCheckResumabilityTest,
     Timestamp resumeTimestamp(100, 1);
     Timestamp oplogTimestamp(100, 2);
 
-    auto shardCheckResumability = createShardCheckResumability(resumeTimestamp);
+    ResumeTokenData tokenData(
+        resumeTimestamp, 0, 0, testUuid(), Value(Document{{"_id"_sd, "ID"_sd}}));
+    auto shardCheckResumability = createShardCheckResumability(tokenData);
     deque<DocumentSource::GetNextResult> mockOplog({Document{{"ts", oplogTimestamp}}});
     getExpCtx()->mongoProcessInterface = std::make_shared<MockMongoInterface>(mockOplog);
     addDocument(resumeTimestamp, "ID");
@@ -525,7 +534,8 @@ TEST_F(ShardCheckResumabilityTest,
 TEST_F(ShardCheckResumabilityTest, ShouldSucceedIfResumeTokenIsPresentAndOplogIsEmpty) {
     Timestamp resumeTimestamp(100, 1);
 
-    auto shardCheckResumability = createShardCheckResumability(resumeTimestamp);
+    ResumeTokenData token(resumeTimestamp, 0, 0, testUuid(), Value(Document{{"_id"_sd, "ID"_sd}}));
+    auto shardCheckResumability = createShardCheckResumability(token);
     deque<DocumentSource::GetNextResult> mockOplog;
     getExpCtx()->mongoProcessInterface = std::make_shared<MockMongoInterface>(mockOplog);
     addDocument(resumeTimestamp, "ID");
@@ -655,5 +665,63 @@ TEST_F(ShardCheckResumabilityTest, ShouldIgnoreOplogAfterFirstEOF) {
     ASSERT_TRUE(result2.isEOF());
 }
 
+TEST_F(ShardCheckResumabilityTest, ShouldSwallowAllEventsAtSameClusterTimeUpToResumeToken) {
+    Timestamp resumeTimestamp(100, 2);
+
+    // Set up the DocumentSourceShardCheckResumability to check for an exact event ResumeToken.
+    ResumeTokenData token(resumeTimestamp, 0, 0, testUuid(), Value(Document{{"_id"_sd, "3"_sd}}));
+    auto shardCheckResumability = createShardCheckResumability(token);
+    deque<DocumentSource::GetNextResult> mockOplog;
+    getExpCtx()->mongoProcessInterface = std::make_shared<MockMongoInterface>(mockOplog);
+
+    // Add 2 events at the same clusterTime as the resume token but whose docKey sort before it.
+    addDocument(resumeTimestamp, "1");
+    addDocument(resumeTimestamp, "2");
+    // Add the resume token, plus one further event whose docKey sorts after the token.
+    addDocument(resumeTimestamp, "3");
+    addDocument(resumeTimestamp, "4");
+
+    // The first event we see should be the resume token...
+    auto result = shardCheckResumability->getNext();
+    ASSERT_TRUE(result.isAdvanced());
+    auto doc = result.getDocument();
+    ASSERT_EQ(token, ResumeToken::parse(doc["_id"].getDocument()).getData());
+    // ... then the post-token event, and then finally EOF.
+    result = shardCheckResumability->getNext();
+    ASSERT_TRUE(result.isAdvanced());
+    Document postResumeTokenDoc{
+        {"_id",
+         ResumeToken({resumeTimestamp, 0, 0, testUuid(), Value(Document{{"_id"_sd, "4"_sd}})})
+             .toDocument(ResumeToken::SerializationFormat::kHexString)}};
+    ASSERT_DOCUMENT_EQ(result.getDocument(), postResumeTokenDoc);
+    ASSERT_TRUE(shardCheckResumability->getNext().isEOF());
+}
+
+TEST_F(ShardCheckResumabilityTest, ShouldSwallowAllEventsAtSameClusterTimePriorToResumeToken) {
+    Timestamp resumeTimestamp(100, 2);
+
+    // Set up the DocumentSourceShardCheckResumability to check for an exact event ResumeToken.
+    ResumeTokenData token(resumeTimestamp, 0, 0, testUuid(), Value(Document{{"_id"_sd, "3"_sd}}));
+    auto shardCheckResumability = createShardCheckResumability(token);
+    deque<DocumentSource::GetNextResult> mockOplog;
+    getExpCtx()->mongoProcessInterface = std::make_shared<MockMongoInterface>(mockOplog);
+
+    // Add 2 events at the same clusterTime as the resume token but whose docKey sort before it.
+    addDocument(resumeTimestamp, "1");
+    addDocument(resumeTimestamp, "2");
+    // Add one further event whose docKey sorts after the token.
+    addDocument(resumeTimestamp, "4");
+
+    // The first event we see should be the post-token event, followed by EOF.
+    auto result = shardCheckResumability->getNext();
+    ASSERT_TRUE(result.isAdvanced());
+    Document postResumeTokenDoc{
+        {"_id",
+         ResumeToken({resumeTimestamp, 0, 0, testUuid(), Value(Document{{"_id"_sd, "4"_sd}})})
+             .toDocument(ResumeToken::SerializationFormat::kHexString)}};
+    ASSERT_DOCUMENT_EQ(result.getDocument(), postResumeTokenDoc);
+    ASSERT_TRUE(shardCheckResumability->getNext().isEOF());
+}
+
 }  // namespace
 }  // namespace mongo
