@@ -79,16 +79,38 @@ std::pair<Value, Value> encodeInBinDataFormat(const ResumeTokenData& data) {
         : Value(BSONBinData(typeBits.getBuffer(), typeBits.getSize(), BinDataType::BinDataGeneral));
     return {Value(rawBinary), typeBitsValue};
 }
+
+// Helper function for makeHighWaterMarkToken and isHighWaterMarkToken.
+ResumeTokenData makeHighWaterMarkResumeTokenData(Timestamp clusterTime,
+                                                 boost::optional<UUID> uuid) {
+    ResumeTokenData tokenData;
+    tokenData.clusterTime = clusterTime;
+    tokenData.tokenType = ResumeTokenData::kHighWaterMarkToken;
+    tokenData.uuid = uuid;
+    return tokenData;
+}
 }  // namespace
 
 bool ResumeTokenData::operator==(const ResumeTokenData& other) const {
-    return clusterTime == other.clusterTime &&
-        (Value::compare(this->documentKey, other.documentKey, nullptr) == 0) && uuid == other.uuid;
+    return clusterTime == other.clusterTime && version == other.version &&
+        tokenType == other.tokenType && applyOpsIndex == other.applyOpsIndex &&
+        fromInvalidate == other.fromInvalidate && uuid == other.uuid &&
+        (Value::compare(this->documentKey, other.documentKey, nullptr) == 0);
 }
 
 std::ostream& operator<<(std::ostream& out, const ResumeTokenData& tokenData) {
-    return out << "{clusterTime: " << tokenData.clusterTime.toString()
-               << "  documentKey: " << tokenData.documentKey << "  uuid: " << tokenData.uuid << "}";
+    out << "{clusterTime: " << tokenData.clusterTime.toString();
+    out << ", version: " << tokenData.version;
+    if (tokenData.version > 0) {
+        out << ", tokenType: " << tokenData.tokenType;
+    }
+    out << ", applyOpsIndex: " << tokenData.applyOpsIndex;
+    if (tokenData.version > 0) {
+        out << ", fromInvalidate: " << static_cast<bool>(tokenData.fromInvalidate);
+    }
+    out << ", uuid: " << tokenData.uuid;
+    out << ", documentKey: " << tokenData.documentKey << "}";
+    return out;
 }
 
 ResumeToken::ResumeToken(const Document& resumeDoc) {
@@ -107,13 +129,19 @@ ResumeToken::ResumeToken(const Document& resumeDoc) {
 }
 
 // We encode the resume token as a KeyString with the sequence:
-// clusterTime, version, applyOpsIndex, uuid, documentKey
-// Only the clusterTime is required.
+// clusterTime, version, applyOpsIndex, fromInvalidate, uuid, documentKey
+// Only the clusterTime, version, applyOpsIndex, and fromInvalidate are required.
 ResumeToken::ResumeToken(const ResumeTokenData& data) {
     BSONObjBuilder builder;
     builder.append("", data.clusterTime);
     builder.append("", data.version);
+    if (data.version >= 1) {
+        builder.appendNumber("", data.tokenType);
+    }
     builder.appendNumber("", data.applyOpsIndex);
+    if (data.version >= 1) {
+        builder.appendBool("", data.fromInvalidate);
+    }
     uassert(50788,
             "Unexpected resume token with a documentKey but no UUID",
             data.uuid || data.documentKey.missing());
@@ -180,6 +208,7 @@ ResumeTokenData ResumeToken::getData() const {
     ResumeTokenData result;
     uassert(40649, "invalid empty resume token", i.more());
     result.clusterTime = i.next().timestamp();
+
     if (!i.more()) {
         // There was nothing other than the timestamp.
         return result;
@@ -195,17 +224,36 @@ ResumeTokenData ResumeToken::getData() const {
         }
         case BSONType::String: {
             // Next comes the resume token version.
+            uassert(50796, "Resume Token does not contain version", i.more());
             auto versionElt = i.next();
-            uassert(50796,
-                    "Resume Token does not contain applyOpsIndex",
+            uassert(50854,
+                    "Invalid resume token: wrong type for version",
                     versionElt.type() == BSONType::NumberInt);
             result.version = versionElt.numberInt();
-            uassert(50795, "Invalid Resume Token: only supports version 0", result.version == 0);
+            uassert(50795,
+                    "Invalid Resume Token: only supports version 0 or 1",
+                    result.version == 0 || result.version == 1);
 
-            // The new format has applyOpsIndex next.
+            if (result.version >= 1) {
+                // The 'tokenType' field was added in version 1 and is not present in v0 tokens.
+                uassert(51055, "Resume Token does not contain tokenType", i.more());
+                auto tokenType = i.next();
+                uassert(51056,
+                        "Resume Token tokenType is not an int.",
+                        tokenType.type() == BSONType::NumberInt);
+                auto typeInt = tokenType.numberInt();
+                uassert(51057,
+                        str::stream() << "Token type " << typeInt << " not recognized",
+                        typeInt == ResumeTokenData::TokenType::kEventToken ||
+                            typeInt == ResumeTokenData::TokenType::kHighWaterMarkToken);
+                result.tokenType = static_cast<ResumeTokenData::TokenType>(typeInt);
+            }
+
+            // Next comes the applyOps index.
+            uassert(50793, "Resume Token does not contain applyOpsIndex", i.more());
             auto applyOpsElt = i.next();
-            uassert(50793,
-                    "Resume Token does not contain applyOpsIndex",
+            uassert(50855,
+                    "Resume Token applyOpsIndex is not an integer",
                     applyOpsElt.type() == BSONType::NumberInt);
             const int applyOpsInd = applyOpsElt.numberInt();
             uassert(50794,
@@ -213,7 +261,18 @@ ResumeTokenData ResumeToken::getData() const {
                     applyOpsInd >= 0);
             result.applyOpsIndex = applyOpsInd;
 
-            // The the UUID and documentKey are not required.
+            if (result.version >= 1) {
+                // The 'fromInvalidate' bool was added in version 1 resume tokens. We don't expect
+                // to see it on version 0. After this bool, the remaining fields should be the same.
+                uassert(50872, "Resume Token does not contain fromInvalidate", i.more());
+                auto fromInvalidate = i.next();
+                uassert(50870,
+                        "Resume Token fromInvalidate is not a boolean.",
+                        fromInvalidate.type() == BSONType::Bool);
+                result.fromInvalidate = ResumeTokenData::FromInvalidate(fromInvalidate.boolean());
+            }
+
+            // The UUID and documentKey are not required.
             if (!i.more()) {
                 return result;
             }
@@ -227,6 +286,7 @@ ResumeTokenData ResumeToken::getData() const {
         }
         default: { MONGO_UNREACHABLE }
     }
+
     uassert(40646, "invalid oversized resume token", !i.more());
     return result;
 }
@@ -265,4 +325,12 @@ ResumeToken ResumeToken::parse(const Document& resumeDoc) {
     return ResumeToken(resumeDoc);
 }
 
+ResumeToken ResumeToken::makeHighWaterMarkToken(Timestamp clusterTime, boost::optional<UUID> uuid) {
+    return ResumeToken(makeHighWaterMarkResumeTokenData(clusterTime, uuid));
+}
+
+bool ResumeToken::isHighWaterMarkToken(const ResumeTokenData& tokenData) {
+    return tokenData == makeHighWaterMarkResumeTokenData(tokenData.clusterTime, tokenData.uuid);
+}
+
 }  // namespace mongo
