@@ -191,7 +191,8 @@ public:
     virtual Status processReplSetGetStatus(BSONObjBuilder* result,
                                            ReplSetGetStatusResponseStyle responseStyle) override;
 
-    virtual void fillIsMasterForReplSet(IsMasterResponse* result) override;
+    virtual void fillIsMasterForReplSet(IsMasterResponse* result,
+                                        const SplitHorizon::Parameters& horizonParams) override;
 
     virtual void appendSlaveInfoData(BSONObjBuilder* result) override;
 
@@ -311,7 +312,7 @@ public:
 
     virtual Status stepUpIfEligible(bool skipDryRun) override;
 
-    virtual Status abortCatchupIfNeeded() override;
+    virtual Status abortCatchupIfNeeded(PrimaryCatchUpConclusionReason reason) override;
 
     void signalDropPendingCollectionsRemovedFromStorage() final;
 
@@ -552,7 +553,7 @@ private:
         // start() can only be called once.
         void start_inlock();
         // Reset the state itself to destruct the state.
-        void abort_inlock();
+        void abort_inlock(PrimaryCatchUpConclusionReason reason);
         // Heartbeat calls this function to update the target optime.
         void signalHeartbeatUpdate_inlock();
 
@@ -847,25 +848,29 @@ private:
      * "originalTerm" was the term during which the dry run began, if the term has since
      * changed, do not run for election.
      */
-    void _processDryRunResult(long long originalTerm);
+    void _processDryRunResult(long long originalTerm,
+                              TopologyCoordinator::StartElectionReason reason);
 
     /**
      * Begins executing a real election. This is called either a successful dry run, or when the
      * dry run was skipped (which may be specified for a ReplSetStepUp).
      */
-    void _startRealElection_inlock(long long originalTerm);
+    void _startRealElection_inlock(long long originalTerm,
+                                   TopologyCoordinator::StartElectionReason reason);
 
     /**
      * Writes the last vote in persistent storage after completing dry run successfully.
      * This job will be scheduled to run in DB worker threads.
      */
     void _writeLastVoteForMyElection(LastVote lastVote,
-                                     const executor::TaskExecutor::CallbackArgs& cbData);
+                                     const executor::TaskExecutor::CallbackArgs& cbData,
+                                     TopologyCoordinator::StartElectionReason reason);
 
     /**
      * Starts VoteRequester to run the real election when last vote write has completed.
      */
-    void _startVoteRequester_inlock(long long newTerm);
+    void _startVoteRequester_inlock(long long newTerm,
+                                    TopologyCoordinator::StartElectionReason reason);
 
     /**
      * Callback called when the VoteRequester has completed; checks the results and
@@ -873,7 +878,8 @@ private:
      * "originalTerm" was the term during which the election began, if the term has since
      * changed, do not step up as primary.
      */
-    void _onVoteRequestComplete(long long originalTerm);
+    void _onVoteRequestComplete(long long originalTerm,
+                                TopologyCoordinator::StartElectionReason reason);
 
     /**
      * Callback called after a random delay, to prevent repeated election ties.
@@ -1248,7 +1254,7 @@ private:
     // Flag that indicates whether writes to databases other than "local" are allowed.  Used to
     // answer canAcceptWritesForDatabase() and canAcceptWritesFor() questions.
     // Always true for standalone nodes.
-    bool _canAcceptNonLocalWrites;  // (GM)
+    AtomicWord<bool> _canAcceptNonLocalWrites;  // (GM)
 
     // Flag that indicates whether reads from databases other than "local" are allowed.  Unlike
     // _canAcceptNonLocalWrites, above, this question is about admission control on secondaries,
