@@ -36,6 +36,7 @@
 
 #include "mongo/base/disallow_copying.h"
 #include "mongo/db/repl/replication_coordinator_impl.h"
+#include "mongo/db/repl/replication_metrics.h"
 #include "mongo/db/repl/topology_coordinator.h"
 #include "mongo/db/repl/vote_requester.h"
 #include "mongo/stdx/mutex.h"
@@ -143,7 +144,7 @@ void ReplicationCoordinatorImpl::_startElectSelfV1_inlock(
     if (reason == TopologyCoordinator::StartElectionReason::kStepUpRequestSkipDryRun) {
         long long newTerm = term + 1;
         log() << "skipping dry run and running for election in term " << newTerm;
-        _startRealElection_inlock(newTerm);
+        _startRealElection_inlock(newTerm, reason);
         lossGuard.dismiss();
         return;
     }
@@ -169,12 +170,15 @@ void ReplicationCoordinatorImpl::_startElectSelfV1_inlock(
     fassert(28685, nextPhaseEvh.getStatus());
     _replExecutor
         ->onEvent(nextPhaseEvh.getValue(),
-                  [=](const executor::TaskExecutor::CallbackArgs&) { _processDryRunResult(term); })
+                  [=](const executor::TaskExecutor::CallbackArgs&) {
+                      _processDryRunResult(term, reason);
+                  })
         .status_with_transitional_ignore();
     lossGuard.dismiss();
 }
 
-void ReplicationCoordinatorImpl::_processDryRunResult(long long originalTerm) {
+void ReplicationCoordinatorImpl::_processDryRunResult(
+    long long originalTerm, TopologyCoordinator::StartElectionReason reason) {
     stdx::lock_guard<stdx::mutex> lk(_mutex);
     LoseElectionDryRunGuardV1 lossGuard(this);
 
@@ -205,11 +209,14 @@ void ReplicationCoordinatorImpl::_processDryRunResult(long long originalTerm) {
     long long newTerm = originalTerm + 1;
     log() << "dry election run succeeded, running for election in term " << newTerm;
 
-    _startRealElection_inlock(newTerm);
+    _startRealElection_inlock(newTerm, reason);
     lossGuard.dismiss();
 }
 
-void ReplicationCoordinatorImpl::_startRealElection_inlock(long long newTerm) {
+void ReplicationCoordinatorImpl::_startRealElection_inlock(
+    long long newTerm, TopologyCoordinator::StartElectionReason reason) {
+    ReplicationMetrics::get(getServiceContext()).incrementNumElectionsCalledForReason(reason);
+
     LoseElectionDryRunGuardV1 lossGuard(this);
 
     TopologyCoordinator::UpdateTermResult updateTermResult;
@@ -226,8 +233,8 @@ void ReplicationCoordinatorImpl::_startRealElection_inlock(long long newTerm) {
     LastVote lastVote{newTerm, _selfIndex};
 
     auto cbStatus = _replExecutor->scheduleWork(
-        [this, lastVote](const executor::TaskExecutor::CallbackArgs& cbData) {
-            _writeLastVoteForMyElection(lastVote, cbData);
+        [this, lastVote, reason](const executor::TaskExecutor::CallbackArgs& cbData) {
+            _writeLastVoteForMyElection(lastVote, cbData, reason);
         });
     if (cbStatus.getStatus() == ErrorCodes::ShutdownInProgress) {
         return;
@@ -237,7 +244,9 @@ void ReplicationCoordinatorImpl::_startRealElection_inlock(long long newTerm) {
 }
 
 void ReplicationCoordinatorImpl::_writeLastVoteForMyElection(
-    LastVote lastVote, const executor::TaskExecutor::CallbackArgs& cbData) {
+    LastVote lastVote,
+    const executor::TaskExecutor::CallbackArgs& cbData,
+    TopologyCoordinator::StartElectionReason reason) {
     // storeLocalLastVoteDocument can call back in to the replication coordinator,
     // so _mutex must be unlocked here.  However, we cannot return until we
     // lock it because we want to lose the election on cancel or error and
@@ -267,13 +276,14 @@ void ReplicationCoordinatorImpl::_writeLastVoteForMyElection(
               << lastVote.getTerm() << ", current term: " << _topCoord->getTerm();
         return;
     }
-    _startVoteRequester_inlock(lastVote.getTerm());
+    _startVoteRequester_inlock(lastVote.getTerm(), reason);
     _replExecutor->signalEvent(_electionDryRunFinishedEvent);
 
     lossGuard.dismiss();
 }
 
-void ReplicationCoordinatorImpl::_startVoteRequester_inlock(long long newTerm) {
+void ReplicationCoordinatorImpl::_startVoteRequester_inlock(
+    long long newTerm, TopologyCoordinator::StartElectionReason reason) {
     const auto lastOpTime = _getMyLastAppliedOpTime_inlock();
 
     _voteRequester.reset(new VoteRequester);
@@ -284,13 +294,17 @@ void ReplicationCoordinatorImpl::_startVoteRequester_inlock(long long newTerm) {
     }
     fassert(28643, nextPhaseEvh.getStatus());
     _replExecutor
-        ->onEvent(
-            nextPhaseEvh.getValue(),
-            [=](const executor::TaskExecutor::CallbackArgs&) { _onVoteRequestComplete(newTerm); })
+        ->onEvent(nextPhaseEvh.getValue(),
+                  [=](const executor::TaskExecutor::CallbackArgs&) {
+                      _onVoteRequestComplete(newTerm, reason);
+                  })
         .status_with_transitional_ignore();
 }
 
-void ReplicationCoordinatorImpl::_onVoteRequestComplete(long long newTerm) {
+MONGO_FAIL_POINT_DEFINE(electionHangsBeforeUpdateMemberState);
+
+void ReplicationCoordinatorImpl::_onVoteRequestComplete(
+    long long newTerm, TopologyCoordinator::StartElectionReason reason) {
     stdx::unique_lock<stdx::mutex> lk(_mutex);
     LoseElectionGuardV1 lossGuard(this);
 
@@ -314,6 +328,8 @@ void ReplicationCoordinatorImpl::_onVoteRequestComplete(long long newTerm) {
             return;
         case VoteRequester::Result::kSuccessfullyElected:
             log() << "election succeeded, assuming primary role in term " << _topCoord->getTerm();
+            ReplicationMetrics::get(getServiceContext())
+                .incrementNumElectionsSuccessfulForReason(reason);
             break;
         case VoteRequester::Result::kPrimaryRespondedNo:
             // This is impossible because we would only require the primary's
@@ -329,9 +345,16 @@ void ReplicationCoordinatorImpl::_onVoteRequestComplete(long long newTerm) {
     _voteRequester.reset();
     auto electionFinishedEvent = _electionFinishedEvent;
 
-    lk.unlock();
-    _performPostMemberStateUpdateAction(kActionWinElection);
+    MONGO_FAIL_POINT_BLOCK(electionHangsBeforeUpdateMemberState, customWait) {
+        auto waitForMillis = Milliseconds(customWait.getData()["waitForMillis"].numberInt());
+        log() << "election succeeded - electionHangsBeforeUpdateMemberState fail point "
+                 "enabled, sleeping "
+              << waitForMillis;
+        sleepFor(waitForMillis);
+    }
 
+    _postWonElectionUpdateMemberState_inlock();
+    lk.unlock();
     _replExecutor->signalEvent(electionFinishedEvent);
     lossGuard.dismiss();
 }
