@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2008-2017 MongoDB, Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -88,6 +90,8 @@ using std::pair;
 
 namespace repl {
 
+MONGO_FAIL_POINT_DEFINE(rollbackExitEarlyAfterCollectionDrop);
+
 using namespace rollback_internal;
 
 bool DocID::operator<(const DocID& other) const {
@@ -948,6 +952,15 @@ Status _syncRollback(OperationContext* opCtx,
             fassert(40497, status);
         });
         syncFixUp(opCtx, how, rollbackSource, replCoord, replicationProcess);
+
+        if (MONGO_FAIL_POINT(rollbackExitEarlyAfterCollectionDrop)) {
+            log() << "rollbackExitEarlyAfterCollectionDrop fail point enabled. Returning early "
+                     "until fail point is disabled.";
+            return Status(ErrorCodes::NamespaceNotFound,
+                          str::stream() << "Failing rollback because "
+                                           "rollbackExitEarlyAfterCollectionDrop fail point "
+                                           "enabled.");
+        }
     } catch (const RSFatalException& e) {
         return Status(ErrorCodes::UnrecoverableRollbackError, e.what());
     }
@@ -1088,17 +1101,27 @@ void rollback_internal::syncFixUp(OperationContext* opCtx,
         invariant(!fixUpInfo.collectionsToResyncMetadata.count(uuid));
 
         NamespaceString nss = UUIDCatalog::get(opCtx).lookupNSSByUUID(uuid);
-        log() << "Dropping collection: " << nss << ", UUID: " << uuid;
-        AutoGetDb dbLock(opCtx, nss.db(), MODE_X);
+        // Do not attempt to acquire the database lock with an empty namespace. We should survive
+        // an attempt to drop a non-existent collection.
+        if (nss.isEmpty()) {
+            log() << "This collection does not exist, UUID: " << uuid;
+        } else {
+            log() << "Dropping collection: " << nss << ", UUID: " << uuid;
+            AutoGetDb dbLock(opCtx, nss.db(), MODE_X);
 
-        Database* db = dbLock.getDb();
-        if (db) {
-            Collection* collection = UUIDCatalog::get(opCtx).lookupCollectionByUUID(uuid);
-            dropCollection(opCtx, nss, collection, db);
-            LOG(1) << "Dropped collection: " << nss << ", UUID: " << uuid;
+            Database* db = dbLock.getDb();
+            if (db) {
+                Collection* collection = UUIDCatalog::get(opCtx).lookupCollectionByUUID(uuid);
+                dropCollection(opCtx, nss, collection, db);
+                LOG(1) << "Dropped collection: " << nss << ", UUID: " << uuid;
+            }
         }
     }
 
+    if (MONGO_FAIL_POINT(rollbackExitEarlyAfterCollectionDrop)) {
+        return;
+    }
+
     // Rolling back renameCollection commands.
     log() << "Rolling back renameCollection commands and collection drop commands.";
 
@@ -1500,6 +1523,12 @@ void rollback(OperationContext* opCtx,
         }
     }
 
+    if (MONGO_FAIL_POINT(rollbackHangAfterTransitionToRollback)) {
+        log() << "rollbackHangAfterTransitionToRollback fail point enabled. Blocking until fail "
+                 "point is disabled (rs_rollback).";
+        MONGO_FAIL_POINT_PAUSE_WHILE_SET(rollbackHangAfterTransitionToRollback);
+    }
+
     try {
         auto status = syncRollback(
             opCtx, localOplog, rollbackSource, requiredRBID, replCoord, replicationProcess);
