@@ -41,6 +41,7 @@
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/shard_metadata_util.h"
 #include "mongo/db/s/sharding_state.h"
+#include "mongo/db/server_options.h"
 #include "mongo/s/catalog/type_shard_collection.h"
 #include "mongo/s/catalog/type_shard_database.h"
 #include "mongo/s/client/shard_registry.h"
@@ -359,7 +360,7 @@ void ShardServerCatalogCacheLoader::notifyOfCollectionVersionUpdate(const Namesp
 }
 
 void ShardServerCatalogCacheLoader::initializeReplicaSetRole(bool isPrimary) {
-    stdx::lock_guard<stdx::mutex> lock(_mutex);
+    stdx::lock_guard<stdx::mutex> lg(_mutex);
     invariant(_role == ReplicaSetRole::None);
 
     if (isPrimary) {
@@ -370,7 +371,7 @@ void ShardServerCatalogCacheLoader::initializeReplicaSetRole(bool isPrimary) {
 }
 
 void ShardServerCatalogCacheLoader::onStepDown() {
-    stdx::lock_guard<stdx::mutex> lock(_mutex);
+    stdx::lock_guard<stdx::mutex> lg(_mutex);
     invariant(_role != ReplicaSetRole::None);
     _contexts.interrupt(ErrorCodes::PrimarySteppedDown);
     ++_term;
@@ -378,58 +379,47 @@ void ShardServerCatalogCacheLoader::onStepDown() {
 }
 
 void ShardServerCatalogCacheLoader::onStepUp() {
-    stdx::lock_guard<stdx::mutex> lock(_mutex);
+    stdx::lock_guard<stdx::mutex> lg(_mutex);
     invariant(_role != ReplicaSetRole::None);
     ++_term;
     _role = ReplicaSetRole::Primary;
 }
 
 std::shared_ptr<Notification<void>> ShardServerCatalogCacheLoader::getChunksSince(
-    const NamespaceString& nss,
-    ChunkVersion version,
-    stdx::function<void(OperationContext*, StatusWith<CollectionAndChangedChunks>)> callbackFn) {
-    long long currentTerm;
-    bool isPrimary;
-    {
-        // Take the mutex so that we can discern whether we're primary or secondary and schedule a
-        // task with the corresponding _term value.
-        stdx::lock_guard<stdx::mutex> lock(_mutex);
-        invariant(_role != ReplicaSetRole::None);
-
-        currentTerm = _term;
-        isPrimary = (_role == ReplicaSetRole::Primary);
-    }
-
+    const NamespaceString& nss, ChunkVersion version, GetChunksSinceCallbackFn callbackFn) {
     auto notify = std::make_shared<Notification<void>>();
 
-    uassertStatusOK(_threadPool.schedule(
-        [ this, nss, version, callbackFn, notify, isPrimary, currentTerm ]() noexcept {
-            auto context = _contexts.makeOperationContext(*Client::getCurrent());
+    bool isPrimary;
+    long long term;
+    std::tie(isPrimary, term) = [&] {
+        stdx::lock_guard<stdx::mutex> lock(_mutex);
+        return std::make_tuple(_role == ReplicaSetRole::Primary, _term);
+    }();
 
-            {
-                stdx::lock_guard<stdx::mutex> lock(_mutex);
-                // We may have missed an OperationContextGroup interrupt since this operation began
-                // but before the OperationContext was added to the group. So we'll check that
-                // we're still in the same _term.
-                if (_term != currentTerm) {
-                    callbackFn(context.opCtx(),
-                               Status{ErrorCodes::Interrupted,
-                                      "Unable to refresh routing table because replica set state "
-                                      "changed or node is shutting down."});
-                    notify->set();
-                    return;
-                }
-            }
+    uassertStatusOK(_threadPool.schedule(
+        [ this, nss, version, callbackFn, notify, isPrimary, term ]() noexcept {
+            auto context = _contexts.makeOperationContext(*Client::getCurrent());
+            auto const opCtx = context.opCtx();
 
             try {
+                {
+                    // We may have missed an OperationContextGroup interrupt since this operation
+                    // began but before the OperationContext was added to the group. So we'll check
+                    // that we're still in the same _term.
+                    stdx::lock_guard<stdx::mutex> lock(_mutex);
+                    uassert(ErrorCodes::Interrupted,
+                            "Unable to refresh routing table because replica set state changed or "
+                            "the node is shutting down.",
+                            _term == term);
+                }
+
                 if (isPrimary) {
-                    _schedulePrimaryGetChunksSince(
-                        context.opCtx(), nss, version, currentTerm, callbackFn, notify);
+                    _schedulePrimaryGetChunksSince(opCtx, nss, version, term, callbackFn, notify);
                 } else {
-                    _runSecondaryGetChunksSince(context.opCtx(), nss, version, callbackFn);
+                    _runSecondaryGetChunksSince(opCtx, nss, version, callbackFn, notify);
                 }
             } catch (const DBException& ex) {
-                callbackFn(context.opCtx(), ex.toStatus());
+                callbackFn(opCtx, ex.toStatus());
                 notify->set();
             }
         }));
@@ -440,44 +430,45 @@ std::shared_ptr<Notification<void>> ShardServerCatalogCacheLoader::getChunksSinc
 void ShardServerCatalogCacheLoader::getDatabase(
     StringData dbName,
     stdx::function<void(OperationContext*, StatusWith<DatabaseType>)> callbackFn) {
-    long long currentTerm;
-    bool isPrimary;
-
-    {
-        // Take the mutex so that we can discern whether we're primary or secondary and schedule a
-        // task with the corresponding _term value.
-        stdx::lock_guard<stdx::mutex> lock(_mutex);
-        invariant(_role != ReplicaSetRole::None);
-
-        currentTerm = _term;
-        isPrimary = (_role == ReplicaSetRole::Primary);
+    // If the FCV version has not been fully upgraded to 4.0, fetch the database entries directly
+    // from the config server, regardless of whether the shard is running as a primary or secondary.
+    // This matches the behaviour of v3.6 and ensures that v4.0 secondaries do not ask a potentially
+    // v3.6 primary to flush its database cache (which is not supported)
+    if (serverGlobalParams.featureCompatibility.getVersion() <
+        ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo40) {
+        _configServerLoader->getDatabase(dbName, callbackFn);
+        return;
     }
 
+    bool isPrimary;
+    long long term;
+    std::tie(isPrimary, term) = [&] {
+        stdx::lock_guard<stdx::mutex> lock(_mutex);
+        return std::make_tuple(_role == ReplicaSetRole::Primary, _term);
+    }();
+
     uassertStatusOK(_threadPool.schedule(
-        [ this, name = dbName.toString(), callbackFn, isPrimary, currentTerm ]() noexcept {
+        [ this, name = dbName.toString(), callbackFn, isPrimary, term ]() noexcept {
             auto context = _contexts.makeOperationContext(*Client::getCurrent());
-
-            {
-                stdx::lock_guard<stdx::mutex> lock(_mutex);
-
-                // We may have missed an OperationContextGroup interrupt since this operation began
-                // but before the OperationContext was added to the group. So we'll check that
-                // we're still in the same _term.
-                if (_term != currentTerm) {
-                    callbackFn(context.opCtx(),
-                               Status{ErrorCodes::Interrupted,
-                                      "Unable to refresh routing table because replica set state "
-                                      "changed or node is shutting down."});
-                    return;
-                }
-            }
+            auto const opCtx = context.opCtx();
 
             try {
+                {
+                    // We may have missed an OperationContextGroup interrupt since this operation
+                    // began but before the OperationContext was added to the group. So we'll check
+                    // that we're still in the same _term.
+                    stdx::lock_guard<stdx::mutex> lock(_mutex);
+                    uassert(
+                        ErrorCodes::InterruptedDueToReplStateChange,
+                        "Unable to refresh database because replica set state changed or the node "
+                        "is shutting down.",
+                        _term == term);
+                }
+
                 if (isPrimary) {
-                    _schedulePrimaryGetDatabase(
-                        context.opCtx(), StringData(name), currentTerm, callbackFn);
+                    _schedulePrimaryGetDatabase(opCtx, name, term, callbackFn);
                 } else {
-                    _runSecondaryGetDatabase(context.opCtx(), StringData(name), callbackFn);
+                    _runSecondaryGetDatabase(opCtx, name, callbackFn);
                 }
             } catch (const DBException& ex) {
                 callbackFn(context.opCtx(), ex.toStatus());
@@ -592,13 +583,15 @@ void ShardServerCatalogCacheLoader::_runSecondaryGetChunksSince(
     OperationContext* opCtx,
     const NamespaceString& nss,
     const ChunkVersion& catalogCacheSinceVersion,
-    stdx::function<void(OperationContext*, StatusWith<CollectionAndChangedChunks>)> callbackFn) {
+    stdx::function<void(OperationContext*, StatusWith<CollectionAndChangedChunks>)> callbackFn,
+    std::shared_ptr<Notification<void>> notify) {
     forcePrimaryCollectionRefreshAndWaitForReplication(opCtx, nss);
 
     // Read the local metadata.
     auto swCollAndChunks =
         _getCompletePersistedMetadataForSecondarySinceVersion(opCtx, nss, catalogCacheSinceVersion);
     callbackFn(opCtx, std::move(swCollAndChunks));
+    notify->set();
 }
 
 void ShardServerCatalogCacheLoader::_schedulePrimaryGetChunksSince(
@@ -690,7 +683,21 @@ void ShardServerCatalogCacheLoader::_schedulePrimaryGetChunksSince(
 
                 swCollectionAndChangedChunks =
                     _getLoaderMetadata(opCtx, nss, catalogCacheSinceVersion, termScheduled);
-                if (swCollectionAndChangedChunks.isOK()) {
+
+                const auto termAfterRefresh = [&] {
+                    stdx::lock_guard<stdx::mutex> lock(_mutex);
+                    return _term;
+                }();
+
+                if (termAfterRefresh != termScheduled) {
+                    // Raising a ConflictingOperationInProgress error here will cause the
+                    // CatalogCache to attempt the refresh as secondary instead of failing the
+                    // operation
+                    swCollectionAndChangedChunks = Status(
+                        ErrorCodes::ConflictingOperationInProgress,
+                        str::stream() << "Replication stepdown occurred during refresh for  '"
+                                      << nss.toString());
+                } else if (swCollectionAndChangedChunks.isOK()) {
                     // After finding metadata remotely, we must have found metadata locally.
                     invariant(!collAndChunks.changedChunks.empty());
                 }
@@ -728,7 +735,7 @@ void ShardServerCatalogCacheLoader::_schedulePrimaryGetDatabase(
         OperationContext * opCtx, StatusWith<DatabaseType> swDatabaseType) {
         if (swDatabaseType == ErrorCodes::NamespaceNotFound) {
             Status scheduleStatus = _ensureMajorityPrimaryAndScheduleDbTask(
-                opCtx, name, dbTask{swDatabaseType, termScheduled});
+                opCtx, name, DBTask{swDatabaseType, termScheduled});
             if (!scheduleStatus.isOK()) {
                 callbackFn(opCtx, scheduleStatus);
                 return;
@@ -739,7 +746,7 @@ void ShardServerCatalogCacheLoader::_schedulePrimaryGetDatabase(
 
         } else if (swDatabaseType.isOK()) {
             Status scheduleStatus = _ensureMajorityPrimaryAndScheduleDbTask(
-                opCtx, name, dbTask{swDatabaseType, termScheduled});
+                opCtx, name, DBTask{swDatabaseType, termScheduled});
             if (!scheduleStatus.isOK()) {
                 callbackFn(opCtx, scheduleStatus);
                 return;
@@ -760,12 +767,12 @@ StatusWith<CollectionAndChangedChunks> ShardServerCatalogCacheLoader::_getLoader
     OperationContext* opCtx,
     const NamespaceString& nss,
     const ChunkVersion& catalogCacheSinceVersion,
-    const long long term) {
+    long long expectedTerm) {
 
     // Get the enqueued metadata first. Otherwise we could miss data between reading persisted and
     // enqueued, if an enqueued task finished after the persisted read but before the enqueued read.
 
-    auto enqueuedRes = _getEnqueuedMetadata(nss, catalogCacheSinceVersion, term);
+    auto enqueuedRes = _getEnqueuedMetadata(nss, catalogCacheSinceVersion, expectedTerm);
     bool tasksAreEnqueued = std::move(enqueuedRes.first);
     CollectionAndChangedChunks enqueued = std::move(enqueuedRes.second);
 
@@ -891,8 +898,7 @@ Status ShardServerCatalogCacheLoader::_ensureMajorityPrimaryAndScheduleCollAndCh
 }
 
 Status ShardServerCatalogCacheLoader::_ensureMajorityPrimaryAndScheduleDbTask(
-    OperationContext* opCtx, StringData dbName, dbTask task) {
-
+    OperationContext* opCtx, StringData dbName, DBTask task) {
     stdx::lock_guard<stdx::mutex> lock(_mutex);
     const bool wasEmpty = _dbTaskLists[dbName.toString()].empty();
     _dbTaskLists[dbName.toString()].addTask(std::move(task));
@@ -1036,7 +1042,7 @@ void ShardServerCatalogCacheLoader::_updatePersistedDbMetadata(OperationContext*
                                                                StringData dbName) {
     stdx::unique_lock<stdx::mutex> lock(_mutex);
 
-    const dbTask& task = _dbTaskLists[dbName.toString()].front();
+    const DBTask& task = _dbTaskLists[dbName.toString()].front();
 
     // If this task is from an old term and no longer valid, do not execute and return true so that
     // the task gets removed from the task list
@@ -1122,7 +1128,7 @@ ShardServerCatalogCacheLoader::collAndChunkTask::collAndChunkTask(
     }
 }
 
-ShardServerCatalogCacheLoader::dbTask::dbTask(StatusWith<DatabaseType> swDatabaseType,
+ShardServerCatalogCacheLoader::DBTask::DBTask(StatusWith<DatabaseType> swDatabaseType,
                                               long long currentTerm)
     : taskNum(taskIdGenerator.fetchAndAdd(1)), termCreated(currentTerm) {
     if (swDatabaseType.isOK()) {
@@ -1166,7 +1172,7 @@ void ShardServerCatalogCacheLoader::CollAndChunkTaskList::addTask(collAndChunkTa
     }
 }
 
-void ShardServerCatalogCacheLoader::DbTaskList::addTask(dbTask task) {
+void ShardServerCatalogCacheLoader::DbTaskList::addTask(DBTask task) {
     if (_tasks.empty()) {
         _tasks.emplace_back(std::move(task));
         return;
