@@ -885,7 +885,7 @@ void execCommandDatabase(OperationContext* opCtx,
                 uassertStatusOK(shardingState->canAcceptShardedCommands());
             }
 
-            behaviors.advanceConfigOptimeFromRequestMetadata(opCtx);
+            behaviors.advanceConfigOpTimeFromRequestMetadata(opCtx);
         }
 
         oss.setAllowImplicitCollectionCreation(allowImplicitCollectionCreationField);
