@@ -68,6 +68,7 @@
 #include "mongo/util/unowned_ptr.h"
 
 namespace mongo {
+
 namespace sorter {
 
 using std::shared_ptr;
@@ -104,19 +105,9 @@ void dassertCompIsSane(const Comparator& comp, const Data& lhs, const Data& rhs)
 #endif
 }
 
-/** Ensures a named file is deleted when this object goes out of scope */
-class FileDeleter {
-public:
-    FileDeleter(const std::string& fileName) : _fileName(fileName) {}
-    ~FileDeleter() {
-        DESTRUCTOR_GUARD(boost::filesystem::remove(_fileName);)
-    }
-
-private:
-    const std::string _fileName;
-};
-
-/** Returns results from sorted in-memory storage */
+/**
+ * Returns results from sorted in-memory storage.
+ */
 template <typename Key, typename Value>
 class InMemIterator : public SortIteratorInterface<Key, Value> {
 public:
@@ -132,6 +123,9 @@ public:
     template <typename Container>
     InMemIterator(const Container& input) : _data(input.begin(), input.end()) {}
 
+    void openSource() {}
+    void closeSource() {}
+
     bool more() {
         return !_data.empty();
     }
@@ -145,7 +139,15 @@ private:
     std::deque<Data> _data;
 };
 
-/** Returns results in order from a single file */
+/**
+ * Returns results from a sorted range within a file. Each instance is given a file name and start
+ * and end offsets.
+ *
+ * This class is NOT responsible for file clean up / deletion. There are openSource() and
+ * closeSource() functions to ensure the FileIterator is not holding the file open when the file is
+ * deleted. Since it is one among many FileIterators, it cannot close a file that may still be in
+ * use elsewhere.
+ */
 template <typename Key, typename Value>
 class FileIterator : public SortIteratorInterface<Key, Value> {
 public:
@@ -155,48 +157,78 @@ public:
     typedef std::pair<Key, Value> Data;
 
     FileIterator(const std::string& fileName,
-                 const Settings& settings,
-                 std::shared_ptr<FileDeleter> fileDeleter)
+                 std::streampos fileStartOffset,
+                 std::streampos fileEndOffset,
+                 const Settings& settings)
         : _settings(settings),
           _done(false),
           _fileName(fileName),
-          _fileDeleter(fileDeleter),
-          _file(_fileName.c_str(), std::ios::in | std::ios::binary) {
-        massert(16814,
-                str::stream() << "error opening file \"" << _fileName << "\": "
-                              << myErrnoWithDescription(),
-                _file.good());
-
-        massert(16815,
+          _fileStartOffset(fileStartOffset),
+          _fileEndOffset(fileEndOffset) {
+        uassert(16815,
                 str::stream() << "unexpected empty file: " << _fileName,
                 boost::filesystem::file_size(_fileName) != 0);
     }
 
+    void openSource() {
+        _file.open(_fileName.c_str(), std::ios::in | std::ios::binary);
+        uassert(16814,
+                str::stream() << "error opening file \"" << _fileName << "\": "
+                              << myErrnoWithDescription(),
+                _file.good());
+        _file.seekg(_fileStartOffset);
+        uassert(50979,
+                str::stream() << "error seeking starting offset of '" << _fileStartOffset
+                              << "' in file \""
+                              << _fileName
+                              << "\": "
+                              << myErrnoWithDescription(),
+                _file.good());
+    }
+
+    void closeSource() {
+        _file.close();
+        uassert(50969,
+                str::stream() << "error closing file \"" << _fileName << "\": "
+                              << myErrnoWithDescription(),
+                !_file.fail());
+    }
+
     bool more() {
         if (!_done)
-            fillIfNeeded();  // may change _done
+            fillBufferIfNeeded();  // may change _done
         return !_done;
     }
 
     Data next() {
         verify(!_done);
-        fillIfNeeded();
+        fillBufferIfNeeded();
 
-        // Note: key must be read before value so can't pass directly to Data constructor
-        auto first = Key::deserializeForSorter(*_reader, _settings.first);
-        auto second = Value::deserializeForSorter(*_reader, _settings.second);
+        // Note: calling read() on the _bufferReader buffer in the deserialize function advances the
+        // buffer. Since Key comes before Value in the _bufferReader, and C++ makes no function
+        // parameter evaluation order guarantees, we cannot deserialize Key and Value straight into
+        // the Data constructor
+        auto first = Key::deserializeForSorter(*_bufferReader, _settings.first);
+        auto second = Value::deserializeForSorter(*_bufferReader, _settings.second);
         return Data(std::move(first), std::move(second));
     }
 
 private:
-    void fillIfNeeded() {
+    /**
+     * Attempts to refill the _bufferReader if it is empty. Expects _done to be false.
+     */
+    void fillBufferIfNeeded() {
         verify(!_done);
 
-        if (!_reader || _reader->atEof())
-            fill();
+        if (!_bufferReader || _bufferReader->atEof())
+            fillBufferFromDisk();
     }
 
-    void fill() {
+    /**
+     * Tries to read from disk and places any results in _bufferReader. If there is no more data to
+     * read, then _done is set to true and the function returns immediately.
+     */
+    void fillBufferFromDisk() {
         int32_t rawSize;
         read(&rawSize, sizeof(rawSize));
         if (_done)
@@ -208,7 +240,7 @@ private:
 
         _buffer.reset(new char[blockSize]);
         read(_buffer.get(), blockSize);
-        massert(16816, "file too short?", !_done);
+        uassert(16816, "file too short?", !_done);
 
         auto encryptionHooks = EncryptionHooks::get(getGlobalServiceContext());
         if (encryptionHooks->enabled()) {
@@ -220,7 +252,7 @@ private:
                                                   reinterpret_cast<uint8_t*>(out.get()),
                                                   blockSize,
                                                   &outLen);
-            massert(28841,
+            uassert(28841,
                     str::stream() << "Failed to unprotect data: " << status.toString(),
                     status.isOK());
             blockSize = outLen;
@@ -228,70 +260,91 @@ private:
         }
 
         if (!compressed) {
-            _reader.reset(new BufReader(_buffer.get(), blockSize));
+            _bufferReader.reset(new BufReader(_buffer.get(), blockSize));
             return;
         }
 
         dassert(snappy::IsValidCompressedBuffer(_buffer.get(), blockSize));
 
         size_t uncompressedSize;
-        massert(17061,
+        uassert(17061,
                 "couldn't get uncompressed length",
                 snappy::GetUncompressedLength(_buffer.get(), blockSize, &uncompressedSize));
 
         std::unique_ptr<char[]> decompressionBuffer(new char[uncompressedSize]);
-        massert(17062,
+        uassert(17062,
                 "decompression failed",
                 snappy::RawUncompress(_buffer.get(), blockSize, decompressionBuffer.get()));
 
         // hold on to decompressed data and throw out compressed data at block exit
         _buffer.swap(decompressionBuffer);
-        _reader.reset(new BufReader(_buffer.get(), uncompressedSize));
+        _bufferReader.reset(new BufReader(_buffer.get(), uncompressedSize));
     }
 
-    // sets _done to true on EOF - asserts on any other error
+    /**
+     * Attempts to read data from disk. Sets _done to true when file offset reaches _fileEndOffset.
+     *
+     * Masserts on any file errors
+     */
     void read(void* out, size_t size) {
-        _file.read(reinterpret_cast<char*>(out), size);
-        if (!_file.good()) {
-            if (_file.eof()) {
-                _done = true;
-                return;
-            }
+        invariant(_file.is_open());
 
-            msgasserted(16817,
-                        str::stream() << "error reading file \"" << _fileName << "\": "
-                                      << myErrnoWithDescription());
+        const std::streampos offset = _file.tellg();
+        uassert(51049,
+                str::stream() << "error reading file \"" << _fileName << "\": "
+                              << myErrnoWithDescription(),
+                offset >= 0);
+
+        if (offset >= _fileEndOffset) {
+            invariant(offset == _fileEndOffset);
+            _done = true;
+            return;
         }
+
+        _file.read(reinterpret_cast<char*>(out), size);
+        uassert(16817,
+                str::stream() << "error reading file \"" << _fileName << "\": "
+                              << myErrnoWithDescription(),
+                _file.good());
         verify(_file.gcount() == static_cast<std::streamsize>(size));
     }
 
     const Settings _settings;
     bool _done;
     std::unique_ptr<char[]> _buffer;
-    std::unique_ptr<BufReader> _reader;
-    std::string _fileName;
-    std::shared_ptr<FileDeleter> _fileDeleter;  // Must outlive _file
+    std::unique_ptr<BufReader> _bufferReader;
+    std::string _fileName;            // File containing the sorted data range.
+    std::streampos _fileStartOffset;  // File offset at which the sorted data range starts.
+    std::streampos _fileEndOffset;    // File offset at which the sorted data range ends.
     std::ifstream _file;
 };
 
-/** Merge-sorts results from 0 or more FileIterators */
+/**
+ * Merge-sorts results from 0 or more FileIterators, all of which should be iterating over sorted
+ * ranges within the same file. This class is given the data source file name upon construction and
+ * is responsible for deleting the data source file upon destruction.
+ */
 template <typename Key, typename Value, typename Comparator>
 class MergeIterator : public SortIteratorInterface<Key, Value> {
 public:
     typedef SortIteratorInterface<Key, Value> Input;
     typedef std::pair<Key, Value> Data;
 
-
     MergeIterator(const std::vector<std::shared_ptr<Input>>& iters,
+                  const std::string& itersSourceFileName,
                   const SortOptions& opts,
                   const Comparator& comp)
         : _opts(opts),
           _remaining(opts.limit ? opts.limit : std::numeric_limits<unsigned long long>::max()),
           _first(true),
-          _greater(comp) {
+          _greater(comp),
+          _itersSourceFileName(itersSourceFileName) {
         for (size_t i = 0; i < iters.size(); i++) {
+            iters[i]->openSource();
             if (iters[i]->more()) {
                 _heap.push_back(std::make_shared<Stream>(i, iters[i]->next(), iters[i]));
+            } else {
+                iters[i]->closeSource();
             }
         }
 
@@ -306,16 +359,22 @@ public:
         _heap.pop_back();
     }
 
+    ~MergeIterator() {
+        // Clear the remaining Stream objects first, to close the file handles before deleting the
+        // file. Some systems will error closing the file if any file handles are still open.
+        _current.reset();
+        _heap.clear();
+        DESTRUCTOR_GUARD(boost::filesystem::remove(_itersSourceFileName));
+    }
+
+    void openSource() {}
+    void closeSource() {}
+
     bool more() {
         if (_remaining > 0 && (_first || !_heap.empty() || _current->more()))
             return true;
 
-        // We are done so clean up resources.
-        // Can't do this in next() due to lifetime guarantees of unowned Data.
-        _heap.clear();
-        _current.reset();
         _remaining = 0;
-
         return false;
     }
 
@@ -331,7 +390,6 @@ public:
 
         if (!_current->advance()) {
             verify(!_heap.empty());
-
             std::pop_heap(_heap.begin(), _heap.end(), _greater);
             _current = _heap.back();
             _heap.pop_back();
@@ -346,11 +404,22 @@ public:
 
 
 private:
-    class Stream {  // Data + Iterator
+    /**
+     * Data iterator over an Input stream.
+     *
+     * This class is responsible for closing the Input source upon destruction, unfortunately,
+     * because that is the path of least resistence to a design change requiring MergeIterator to
+     * handle eventual deletion of said Input source.
+     */
+    class Stream {
     public:
         Stream(size_t fileNum, const Data& first, std::shared_ptr<Input> rest)
             : fileNum(fileNum), _current(first), _rest(rest) {}
 
+        ~Stream() {
+            _rest->closeSource();
+        }
+
         const Data& current() const {
             return _current;
         }
@@ -396,6 +465,7 @@ private:
     std::shared_ptr<Stream> _current;
     std::vector<std::shared_ptr<Stream>> _heap;  // MinHeap
     STLComparator _greater;                      // named so calls make sense
+    std::string _itersSourceFileName;
 };
 
 template <typename Key, typename Value, typename Comparator>
@@ -412,9 +482,22 @@ public:
                   const Settings& settings = Settings())
         : _comp(comp), _settings(settings), _opts(opts), _memUsed(0) {
         verify(_opts.limit == 0);
+        if (_opts.extSortAllowed) {
+            _fileName = _opts.tempDir + "/" + nextFileName();
+        }
+    }
+
+    ~NoLimitSorter() {
+        if (!_done) {
+            // If done() was never called to return a MergeIterator, then this Sorter still owns
+            // file deletion.
+            DESTRUCTOR_GUARD(boost::filesystem::remove(_fileName));
+        }
     }
 
     void add(const Key& key, const Value& val) {
+        invariant(!_done);
+
         _data.push_back(std::make_pair(key, val));
 
         _memUsed += key.memUsageForSorter();
@@ -425,21 +508,17 @@ public:
     }
 
     Iterator* done() {
+        invariant(!_done);
+
         if (_iters.empty()) {
             sort();
             return new InMemIterator<Key, Value>(_data);
         }
 
         spill();
-        return Iterator::merge(_iters, _opts, _comp);
-    }
-
-    // TEMP these are here for compatibility. Will be replaced with a general stats API
-    int numFiles() const {
-        return _iters.size();
-    }
-    size_t memUsed() const {
-        return _memUsed;
+        Iterator* mergeIt = Iterator::merge(_iters, _fileName, _opts, _comp);
+        _done = true;
+        return mergeIt;
     }
 
 private:
@@ -465,6 +544,8 @@ private:
     }
 
     void spill() {
+        invariant(!_done);
+
         if (_data.empty())
             return;
 
@@ -483,12 +564,15 @@ private:
 
         sort();
 
-        SortedFileWriter<Key, Value> writer(_opts, _settings);
+        SortedFileWriter<Key, Value> writer(
+            _opts, _fileName, _nextSortedFileWriterOffset, _settings);
         for (; !_data.empty(); _data.pop_front()) {
             writer.addAlreadySorted(_data.front().first, _data.front().second);
         }
+        Iterator* iteratorPtr = writer.done();
+        _nextSortedFileWriterOffset = writer.getFileEndOffset();
 
-        _iters.push_back(std::shared_ptr<Iterator>(writer.done()));
+        _iters.push_back(std::shared_ptr<Iterator>(iteratorPtr));
 
         _memUsed = 0;
     }
@@ -496,6 +580,9 @@ private:
     const Comparator _comp;
     const Settings _settings;
     SortOptions _opts;
+    std::string _fileName;
+    std::streampos _nextSortedFileWriterOffset = 0;
+    bool _done = false;
     size_t _memUsed;
     std::deque<Data> _data;                         // the "current" data
     std::vector<std::shared_ptr<Iterator>> _iters;  // data that has already been spilled
@@ -536,14 +623,6 @@ public:
         }
     }
 
-    // TEMP these are here for compatibility. Will be replaced with a general stats API
-    int numFiles() const {
-        return 0;
-    }
-    size_t memUsed() const {
-        return _best.first.memUsageForSorter() + _best.second.memUsageForSorter();
-    }
-
 private:
     const Comparator _comp;
     Data _best;
@@ -572,6 +651,10 @@ public:
         // This also *works* with limit==1 but LimitOneSorter should be used instead
         verify(_opts.limit > 1);
 
+        if (_opts.extSortAllowed) {
+            _fileName = _opts.tempDir + "/" + nextFileName();
+        }
+
         // Preallocate a fixed sized vector of the required size if we
         // don't expect it to have a major impact on our memory budget.
         // This is the common case with small limits.
@@ -580,7 +663,17 @@ public:
         }
     }
 
+    ~TopKSorter() {
+        if (!_done) {
+            // If done() was never called to return a MergeIterator, then this Sorter still owns
+            // file deletion.
+            DESTRUCTOR_GUARD(boost::filesystem::remove(_fileName));
+        }
+    }
+
     void add(const Key& key, const Value& val) {
+        invariant(!_done);
+
         STLComparator less(_comp);
         Data contender(key, val);
 
@@ -630,15 +723,9 @@ public:
         }
 
         spill();
-        return Iterator::merge(_iters, _opts, _comp);
-    }
-
-    // TEMP these are here for compatibility. Will be replaced with a general stats API
-    int numFiles() const {
-        return _iters.size();
-    }
-    size_t memUsed() const {
-        return _memUsed;
+        Iterator* iterator = Iterator::merge(_iters, _fileName, _opts, _comp);
+        _done = true;
+        return iterator;
     }
 
 private:
@@ -743,6 +830,8 @@ private:
     }
 
     void spill() {
+        invariant(!_done);
+
         if (_data.empty())
             return;
 
@@ -765,7 +854,8 @@ private:
         sort();
         updateCutoff();
 
-        SortedFileWriter<Key, Value> writer(_opts, _settings);
+        SortedFileWriter<Key, Value> writer(
+            _opts, _fileName, _nextSortedFileWriterOffset, _settings);
         for (size_t i = 0; i < _data.size(); i++) {
             writer.addAlreadySorted(_data[i].first, _data[i].second);
         }
@@ -773,7 +863,9 @@ private:
         // clear _data and release backing array's memory
         std::vector<Data>().swap(_data);
 
-        _iters.push_back(std::shared_ptr<Iterator>(writer.done()));
+        Iterator* iteratorPtr = writer.done();
+        _nextSortedFileWriterOffset = writer.getFileEndOffset();
+        _iters.push_back(std::shared_ptr<Iterator>(iteratorPtr));
 
         _memUsed = 0;
     }
@@ -781,6 +873,9 @@ private:
     const Comparator _comp;
     const Settings _settings;
     SortOptions _opts;
+    std::string _fileName;
+    std::streampos _nextSortedFileWriterOffset = 0;
+    bool _done = false;
     size_t _memUsed;
     std::vector<Data> _data;  // the "current" data. Organized as max-heap if size == limit.
     std::vector<std::shared_ptr<Iterator>> _iters;  // data that has already been spilled
@@ -794,46 +889,44 @@ private:
     size_t _medianCount;  // Number of docs better or equal to _lastMedian kept so far.
 };
 
-inline unsigned nextFileNumber() {
-    // This is unified across all Sorter types and instances.
-    static AtomicUInt32 fileCounter;
-    return fileCounter.fetchAndAdd(1);
-}
 }  // namespace sorter
 
 //
 // SortedFileWriter
 //
 
-
 template <typename Key, typename Value>
-SortedFileWriter<Key, Value>::SortedFileWriter(const SortOptions& opts, const Settings& settings)
+SortedFileWriter<Key, Value>::SortedFileWriter(const SortOptions& opts,
+                                               const std::string& fileName,
+                                               const std::streampos fileStartOffset,
+                                               const Settings& settings)
     : _settings(settings) {
     namespace str = mongoutils::str;
 
     // This should be checked by consumers, but if we get here don't allow writes.
-    massert(
+    uassert(
         16946, "Attempting to use external sort from mongos. This is not allowed.", !isMongos());
 
-    massert(17148,
+    uassert(17148,
             "Attempting to use external sort without setting SortOptions::tempDir",
             !opts.tempDir.empty());
 
-    {
-        StringBuilder sb;
-        sb << opts.tempDir << "/extsort." << sorter::nextFileNumber();
-        _fileName = sb.str();
-    }
-
     boost::filesystem::create_directories(opts.tempDir);
 
-    _file.open(_fileName.c_str(), std::ios::binary | std::ios::out);
-    massert(16818,
+    _fileName = fileName;
+
+    // We open the provided file in append mode so that SortedFileWriter instances can share the
+    // same file, used serially. We want to share files in order to stay below system open file
+    // limits.
+    _file.open(_fileName.c_str(), std::ios::binary | std::ios::app | std::ios::out);
+    uassert(16818,
             str::stream() << "error opening file \"" << _fileName << "\": "
                           << sorter::myErrnoWithDescription(),
             _file.good());
-
-    _fileDeleter = std::make_shared<sorter::FileDeleter>(_fileName);
+    // The file descriptor is positioned at the end of a file when opened in append mode, but
+    // _file.tellp() is not initialized on all systems to reflect this. Therefore, we must also pass
+    // in the expected offset to this constructor.
+    _fileStartOffset = fileStartOffset;
 
     // throw on failure
     _file.exceptions(std::ios::failbit | std::ios::badbit | std::ios::eofbit);
@@ -879,7 +972,7 @@ void SortedFileWriter<Key, Value>::spill() {
                                                         reinterpret_cast<uint8_t*>(out.get()),
                                                         protectedSizeMax,
                                                         &resultLen);
-        massert(28842,
+        uassert(28842,
                 str::stream() << "Failed to compress data: " << status.toString(),
                 status.isOK());
         outBuffer = out.get();
@@ -891,7 +984,6 @@ void SortedFileWriter<Key, Value>::spill() {
     try {
         _file.write(reinterpret_cast<const char*>(&size), sizeof(size));
         _file.write(outBuffer, std::abs(size));
-
     } catch (const std::exception&) {
         msgasserted(16821,
                     str::stream() << "error writing to file \"" << _fileName << "\": "
@@ -904,8 +996,20 @@ void SortedFileWriter<Key, Value>::spill() {
 template <typename Key, typename Value>
 SortIteratorInterface<Key, Value>* SortedFileWriter<Key, Value>::done() {
     spill();
+    std::streampos currentFileOffset = _file.tellp();
+    uassert(50980,
+            str::stream() << "error fetching current file descriptor offset in file \"" << _fileName
+                          << "\": "
+                          << sorter::myErrnoWithDescription(),
+            currentFileOffset >= 0);
+
+    // In case nothing was written to disk, use _fileStartOffset because tellp() may not be
+    // initialized on all systems upon opening the file.
+    _fileEndOffset = currentFileOffset < _fileStartOffset ? _fileStartOffset : currentFileOffset;
     _file.close();
-    return new sorter::FileIterator<Key, Value>(_fileName, _settings, _fileDeleter);
+
+    return new sorter::FileIterator<Key, Value>(
+        _fileName, _fileStartOffset, _fileEndOffset, _settings);
 }
 
 //
@@ -916,9 +1020,10 @@ template <typename Key, typename Value>
 template <typename Comparator>
 SortIteratorInterface<Key, Value>* SortIteratorInterface<Key, Value>::merge(
     const std::vector<std::shared_ptr<SortIteratorInterface>>& iters,
+    const std::string& fileName,
     const SortOptions& opts,
     const Comparator& comp) {
-    return new sorter::MergeIterator<Key, Value, Comparator>(iters, opts, comp);
+    return new sorter::MergeIterator<Key, Value, Comparator>(iters, fileName, opts, comp);
 }
 
 template <typename Key, typename Value>
@@ -927,11 +1032,11 @@ Sorter<Key, Value>* Sorter<Key, Value>::make(const SortOptions& opts,
                                              const Comparator& comp,
                                              const Settings& settings) {
     // This should be checked by consumers, but if it isn't try to fail early.
-    massert(16947,
+    uassert(16947,
             "Attempting to use external sort from mongos. This is not allowed.",
             !(isMongos() && opts.extSortAllowed));
 
-    massert(17149,
+    uassert(17149,
             "Attempting to use external sort without setting SortOptions::tempDir",
             !(opts.extSortAllowed && opts.tempDir.empty()));
 
