@@ -38,7 +38,6 @@
 #include "mongo/base/status.h"
 #include "mongo/bson/timestamp.h"
 #include "mongo/db/repl/read_concern_level.h"
-#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/storage/snapshot.h"
 
 namespace mongo {
@@ -46,6 +45,36 @@ namespace mongo {
 class BSONObjBuilder;
 class OperationContext;
 
+/**
+ * Storage statistics management class, with interfaces to provide the statistics in the BSON format
+ * and an operator to add the statistics values.
+ */
+class StorageStats {
+    MONGO_DISALLOW_COPYING(StorageStats);
+
+public:
+    StorageStats() = default;
+
+    virtual ~StorageStats(){};
+
+    /**
+     * Provides the storage statistics in the form of a BSONObj.
+     */
+    virtual BSONObj toBSON() = 0;
+
+    /**
+     * Add the statistics values.
+     */
+    virtual StorageStats& operator+=(const StorageStats&) = 0;
+
+    /**
+     * Provides the ability to create an instance of this class outside of the storage integration
+     * layer.
+     */
+    virtual std::shared_ptr<StorageStats> getCopy() = 0;
+};
+
+
 /**
  * A RecoveryUnit is responsible for ensuring that data is persisted.
  * All on-disk information must be mutated through this interface.
@@ -213,6 +242,13 @@ public:
                   "This storage engine does not support prepared transactions");
     }
 
+    /**
+     * Fetches the storage level statistics.
+     */
+    virtual std::shared_ptr<StorageStats> getOperationStatistics() const {
+        return (nullptr);
+    }
+
     /**
      * When no read timestamp is provided to the recovery unit, the ReadSource indicates which
      * external timestamp source to read from.
