@@ -102,12 +102,15 @@ public:
             }
         }
         const bool ephemeral = false;
+        const auto maxCacheOverflowMB =
+            static_cast<size_t>(1024 * wiredTigerGlobalOptions.maxCacheOverflowFileSizeGB);
         WiredTigerKVEngine* kv =
             new WiredTigerKVEngine(getCanonicalName().toString(),
                                    params.dbpath,
                                    getGlobalServiceContext()->getFastClockSource(),
                                    wiredTigerGlobalOptions.engineConfig,
                                    cacheMB,
+                                   maxCacheOverflowMB,
                                    params.dur,
                                    ephemeral,
                                    params.repair,
@@ -117,6 +120,8 @@ public:
         // Intentionally leaked.
         new WiredTigerServerStatusSection(kv);
         new WiredTigerEngineRuntimeConfigParameter(kv);
+        new WiredTigerMaxCacheOverflowSizeGBParameter(
+            kv, wiredTigerGlobalOptions.maxCacheOverflowFileSizeGB);
 
         KVStorageEngineOptions options;
         options.directoryPerDB = params.directoryperdb;
