@@ -253,14 +253,14 @@ public:
 
     static Status setTableLogging(WT_SESSION* session, const std::string& uri, bool on);
 
-private:
     /**
      * Casts unsigned 64-bit statistics value to T.
      * If original value exceeds maximum value of T, return max(T).
      */
     template <typename T>
-    static T _castStatisticsValue(uint64_t statisticsValue);
+    static T castStatisticsValue(uint64_t statisticsValue);
 
+private:
     /**
      * Casts unsigned 64-bit statistics value to T.
      * If original value exceeds 'maximumResultType', return 'maximumResultType'.
@@ -326,7 +326,7 @@ StatusWith<ResultType> WiredTigerUtil::getStatisticsValueAs(WT_SESSION* session,
 
 // static
 template <typename ResultType>
-ResultType WiredTigerUtil::_castStatisticsValue(uint64_t statisticsValue) {
+ResultType WiredTigerUtil::castStatisticsValue(uint64_t statisticsValue) {
     return _castStatisticsValue<ResultType>(statisticsValue,
                                             std::numeric_limits<ResultType>::max());
 }
