@@ -53,7 +53,9 @@ public:
         : _reactor(std::move(reactor)), _tl(tl), _onConnectHook(std::move(onConnectHook)) {}
 
     std::shared_ptr<ConnectionPool::ConnectionInterface> makeConnection(
-        const HostAndPort& hostAndPort, size_t generation) override;
+        const HostAndPort& hostAndPort,
+        transport::ConnectSSLMode sslMode,
+        size_t generation) override;
     std::shared_ptr<ConnectionPool::TimerInterface> makeTimer() override;
 
     Date_t now() override;
@@ -110,6 +112,7 @@ public:
 
     void setTimeout(Milliseconds timeout, TimeoutCallback cb) override;
     void cancelTimeout() override;
+    Date_t now() override;
 
 private:
     transport::ReactorHandle _reactor;
@@ -122,14 +125,16 @@ public:
                  transport::ReactorHandle reactor,
                  ServiceContext* serviceContext,
                  HostAndPort peer,
+                 transport::ConnectSSLMode sslMode,
                  size_t generation,
                  NetworkConnectionHook* onConnectHook)
-        : TLTypeFactory::Type(factory),
+        : ConnectionInterface(generation),
+          TLTypeFactory::Type(factory),
           _reactor(reactor),
           _serviceContext(serviceContext),
           _timer(factory->makeTimer()),
           _peer(std::move(peer)),
-          _generation(generation),
+          _sslMode(sslMode),
           _onConnectHook(onConnectHook) {}
     ~TLConnection() {
         // Release must be the first expression of this dtor
@@ -140,37 +145,28 @@ public:
         cancelAsync();
     }
 
-    void indicateSuccess() override;
-    void indicateFailure(Status status) override;
-    void indicateUsed() override;
     const HostAndPort& getHostAndPort() const override;
+    transport::ConnectSSLMode getSslMode() const override;
     bool isHealthy() override;
     AsyncDBClient* client();
+    Date_t now() override;
 
 private:
-    Date_t getLastUsed() const override;
-    const Status& getStatus() const override;
-
     void setTimeout(Milliseconds timeout, TimeoutCallback cb) override;
     void cancelTimeout() override;
     void setup(Milliseconds timeout, SetupCallback cb) override;
-    void resetToUnknown() override;
     void refresh(Milliseconds timeout, RefreshCallback cb) override;
     void cancelAsync();
 
-    size_t getGeneration() const override;
-
 private:
     transport::ReactorHandle _reactor;
     ServiceContext* const _serviceContext;
     std::shared_ptr<ConnectionPool::TimerInterface> _timer;
 
     HostAndPort _peer;
-    size_t _generation;
+    transport::ConnectSSLMode _sslMode;
     NetworkConnectionHook* const _onConnectHook;
     AsyncDBClient::Handle _client;
-    Date_t _lastUsed;
-    Status _status = ConnectionPool::kConnectionStateUnknown;
 };
 
 }  // namespace connection_pool_asio
