@@ -69,6 +69,8 @@ public:
     static const BSONField<BSONArray> tags;
     static const BSONField<ShardState> state;
 
+    ShardType() = default;
+    ShardType(std::string name, std::string host, std::vector<std::string> tags = {});
 
     /**
      * Constructs a new ShardType object from BSON.
