@@ -122,13 +122,32 @@ repl::OpTime Grid::configOpTime() const {
     return _configOpTime;
 }
 
-void Grid::advanceConfigOpTime(repl::OpTime opTime) {
+boost::optional<repl::OpTime> Grid::advanceConfigOpTime(OperationContext* opCtx,
+                                                        repl::OpTime opTime,
+                                                        StringData what) {
+    const auto prevOpTime = _advanceConfigOpTime(opTime);
+    if (prevOpTime && prevOpTime->getTerm() != opTime.getTerm()) {
+        std::string clientAddr = "(unknown)";
+        if (opCtx && opCtx->getClient()) {
+            clientAddr = opCtx->getClient()->clientAddress(true);
+        }
+        log() << "Received " << what << " " << clientAddr << " indicating config server optime "
+                                                             "term has increased, previous optime "
+              << *prevOpTime << ", now " << opTime;
+    }
+    return prevOpTime;
+}
+
+boost::optional<repl::OpTime> Grid::_advanceConfigOpTime(const repl::OpTime& opTime) {
     invariant(serverGlobalParams.clusterRole != ClusterRole::ConfigServer);
 
     stdx::lock_guard<stdx::mutex> lk(_mutex);
     if (_configOpTime < opTime) {
+        repl::OpTime prev = _configOpTime;
         _configOpTime = opTime;
+        return prev;
     }
+    return boost::none;
 }
 
 void Grid::clearForUnitTests() {
