@@ -154,9 +154,12 @@ public:
     /**
      * Called whenever a mongos or shard gets a response from a config server or shard and updates
      * what we've seen as the last config server optime.
+     * If the config optime was updated, returns the previous value.
      * NOTE: This is not valid to call on a config server instance.
      */
-    void advanceConfigOpTime(repl::OpTime opTime);
+    boost::optional<repl::OpTime> advanceConfigOpTime(OperationContext* opCtx,
+                                                      repl::OpTime opTime,
+                                                      StringData what);
 
     /**
      * Clears the grid object so that it can be reused between test executions. This will not
@@ -195,6 +198,13 @@ private:
     // This value is updated any time a shard or mongos talks to a config server or a shard.
     repl::OpTime _configOpTime;
 
+    /**
+     * Called to update what we've seen as the last config server optime.
+     * If the config optime was updated, returns the previous value.
+     * NOTE: This is not valid to call on a config server instance.
+     */
+    boost::optional<repl::OpTime> _advanceConfigOpTime(const repl::OpTime& opTime);
+
     // Deprecated. This is only used on mongos, and once addShard is solely handled by the configs,
     // it can be deleted.
     // Can 'localhost' be used in shard addresses?
