@@ -181,22 +181,30 @@ assert = (function() {
             msg, "[" + tojson(a) + "] != [" + tojson(b) + "] are not equal"));
     };
 
-    assert.docEq = function(a, b, msg) {
-        _validateAssertionMessage(msg);
-
+    function _isDocEq(a, b) {
         if (a == b) {
-            return;
+            return true;
         }
 
         var aSorted = sortDoc(a);
         var bSorted = sortDoc(b);
 
         if ((aSorted != null && bSorted != null) && friendlyEqual(aSorted, bSorted)) {
+            return true;
+        }
+
+        return false;
+    }
+
+    assert.docEq = function(a, b, msg) {
+        _validateAssertionMessage(msg);
+
+        if (_isDocEq(a, b)) {
             return;
         }
 
         doassert(_buildAssertionMessage(
-            msg, "[" + tojson(aSorted) + "] != [" + tojson(bSorted) + "] are not equal"));
+            msg, "[" + tojson(a) + "] != [" + tojson(b) + "] are not equal"));
     };
 
     assert.setEq = function(aSet, bSet, msg) {
@@ -213,6 +221,42 @@ assert = (function() {
         }
     };
 
+    /**
+     * Throws if the two arrays do not have the same members, in any order. By default, nested
+     * arrays must have the same order to be considered equal.
+     *
+     * Optionally accepts a compareFn to compare values instead of using docEq.
+     */
+    assert.sameMembers = function(aArr, bArr, msg, compareFn = _isDocEq) {
+        _validateAssertionMessage(msg);
+
+        const failAssertion = function() {
+            doassert(_buildAssertionMessage(msg, tojson(aArr) + " != " + tojson(bArr)));
+        };
+
+        if (aArr.length !== bArr.length) {
+            failAssertion();
+        }
+
+        // Keep a set of which indices we've already used to avoid double counting values.
+        const matchedIndicesInRight = new Set();
+        for (let a of aArr) {
+            let foundMatch = false;
+            for (let i = 0; i < bArr.length; ++i) {
+                // Sort both inputs in case either is a document. Note: by default this does not
+                // sort any nested arrays.
+                if (!matchedIndicesInRight.has(i) && compareFn(a, bArr[i])) {
+                    matchedIndicesInRight.add(i);
+                    foundMatch = true;
+                    break;
+                }
+            }
+            if (!foundMatch) {
+                failAssertion();
+            }
+        }
+    };
+
     assert.eq.automsg = function(a, b) {
         assert.eq(eval(a), eval(b), "[" + a + "] != [" + b + "]");
     };
@@ -616,6 +660,15 @@ assert = (function() {
         return res;
     }
 
+    assert.commandWorkedOrFailedWithCode = function commandWorkedOrFailedWithCode(
+        res, errorCodeSet, msg) {
+        if (!res.ok) {
+            return assert.commandFailedWithCode(res, errorCodeSet, msg);
+        } else {
+            return assert.commandWorked(res, msg);
+        }
+    };
+
     assert.commandWorked = function(res, msg) {
         return _assertCommandWorked(res, msg, {ignoreWriteErrors: false});
     };
@@ -938,5 +991,16 @@ assert = (function() {
         }
     };
 
+    assert.includes = function(haystack, needle, msg) {
+        if (!haystack.includes(needle)) {
+            var assertMsg = "string [" + haystack + "] does not include [" + needle + "]";
+            if (msg) {
+                assertMsg += ", " + msg;
+            }
+
+            doassert(assertMsg);
+        }
+    };
+
     return assert;
 })();
