@@ -981,6 +981,12 @@ int _main(int argc, char* argv[], char** envp) {
                         false,
                         true,
                         false);
+
+            scope->exec("shellHelper( 'show', 'nonGenuineMongoDBCheck' )",
+                        "(nonGenuineMongoDBCheck)",
+                        false,
+                        true,
+                        false);
         }
 
         shellHistoryInit();
