@@ -54,11 +54,11 @@ struct ShellGlobalParams {
 
     std::string username;
     std::string password;
-    bool usingPassword;
     std::string authenticationMechanism;
     std::string authenticationDatabase;
     std::string gssapiServiceName;
     std::string gssapiHostName;
+    std::string networkMessageCompressors;
 
     bool runShell;
     bool nodb;
