@@ -78,7 +78,6 @@ MONGO_REGISTER_SHIM(BenchRunConfig::createConnectionImpl)
 namespace shell_utils {
 
 std::string _dbConnect;
-std::string _dbAuth;
 
 const char* argv0 = 0;
 void RecordMyLocation(const char* _argv0) {
@@ -258,7 +257,8 @@ BSONObj replMonitorStats(const BSONObj& a, void* data) {
             "replMonitorStats requires a single string argument (the ReplSet name)",
             a.nFields() == 1 && a.firstElement().type() == String);
 
-    ReplicaSetMonitorPtr rsm = ReplicaSetMonitor::get(a.firstElement().valuestrsafe());
+    auto name = a.firstElement().valuestrsafe();
+    ReplicaSetMonitorPtr rsm = ReplicaSetMonitor::get(name);
     if (!rsm) {
         return BSON(""
                     << "no ReplSetMonitor exists by that name");
@@ -266,7 +266,8 @@ BSONObj replMonitorStats(const BSONObj& a, void* data) {
 
     BSONObjBuilder result;
     rsm->appendInfo(result);
-    return result.obj();
+    // Stats are like {replSetName: {hosts: [{ ... }, { ... }]}}.
+    return result.obj()[name].Obj().getOwned();
 }
 
 BSONObj useWriteCommandsDefault(const BSONObj& a, void* data) {
@@ -344,9 +345,6 @@ void initScope(Scope& scope) {
     if (!_dbConnect.empty()) {
         uassert(12513, "connect failed", scope.exec(_dbConnect, "(connect)", false, true, false));
     }
-    if (!_dbAuth.empty()) {
-        uassert(12514, "login failed", scope.exec(_dbAuth, "(auth)", true, true, false));
-    }
 }
 
 Prompter::Prompter(const string& prompt) : _prompt(prompt), _confirmed() {}
